from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any

supported_mysql_engines = ["5.6", "5.7", "8", "8.0"]
supported_sql_engines = ["08r2_ent_ha", "2012_ent_ha", "2016_ent_ha", "2017_ent", "2019_std_ha", "2019_ent"]


class RDSTransparentDataEncryptionEnabled(BaseResourceValueCheck):
    def __init__(self):
        """
        Check valid db engines here: https://www.alibabacloud.com/help/en/apsaradb-for-rds/latest/create-an-instance
        """

        name = "Ensure Transparent Data Encryption is Enabled on instance"
        id = "CKV_ALI_22"
        supported_resources = ['alicloud_db_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("engine") == ["MySQL"] or conf.get("engine") == ["SQLServer"]:
            if conf.get("engine_version") and isinstance(conf.get("engine_version"), list):
                if conf.get("engine_version")[0] in supported_mysql_engines or \
                        conf.get("engine_version")[0] in supported_sql_engines:
                    if conf.get("tde_status") == ["Enabled"]:
                        return CheckResult.PASSED
                    self.evaluated_keys = ["engine_version"]
                    return CheckResult.FAILED
        return CheckResult.UNKNOWN

    def get_inspected_key(self):
        return "tde_status"

    def get_expected_value(self) -> Any:
        return "Enabled"


check = RDSTransparentDataEncryptionEnabled()



from __future__ import annotations

import logging
from collections.abc import Hashable
from pathlib import Path
from typing import Tuple, Dict, Any, List, TYPE_CHECKING

import yaml
from yaml.loader import SafeLoader

from checkov.common.util.file_utils import read_file_with_any_encoding

if TYPE_CHECKING:
    from yaml import MappingNode

logger = logging.getLogger(__name__)


def loads(content: str) -> list[dict[str, Any]]:
    """
    Load the given YAML string
    """

    content = "[" + content + "]"
    content = content.replace('}{', '},{')
    content = content.replace('}\n{', '},\n{')

    template: list[dict[str, Any]] = yaml.load(content, Loader=SafeLineLoader)  # nosec  # custom safe loader

    # Convert an empty file to an empty dict
    if template is None:
        template = []

    return template


def load(filename: Path) -> Tuple[List[Dict[str, Any]], List[Tuple[int, str]]]:
    """
    Load the given JSON file
    """

    content = read_file_with_any_encoding(file_path=filename)

    if not all(key in content for key in ("apiVersion", "kind")):
        return [{}], []

    file_lines = [(idx + 1, line) for idx, line in enumerate(content.splitlines(keepends=True))]

    template = loads(content)

    return template, file_lines


class SafeLineLoader(SafeLoader):
    def construct_mapping(self, node: MappingNode, deep: bool = False) -> dict[Hashable, Any]:
        mapping = super().construct_mapping(node, deep=deep)
        # Add 1 so line numbering starts at 1
        # mapping['__line__'] = node.start_mark.line + 1
        mapping['__startline__'] = node.start_mark.line + 1
        mapping['__endline__'] = node.end_mark.line + 1
        return mapping



import os
import tempfile
from pathlib import Path

from checkov.common.models.enums import CheckFailLevel
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.http_utils import normalize_bc_url
from checkov.common.util.type_forcers import convert_str_to_bool, force_int
from checkov.version import version


class EnvVarsConfig:
    def __init__(self) -> None:
        self.BC_API_URL = normalize_bc_url(os.getenv("BC_API_URL", "https://www.bridgecrew.cloud"))
        self.BC_ENABLE_PERSIST_GRAPHS = convert_str_to_bool(os.getenv("BC_ENABLE_PERSIST_GRAPHS", True))
        self.BC_PERSIST_GRAPHS_TIMEOUT = force_int(os.getenv("BC_PERSIST_GRAPHS_TIMEOUT", 60))
        self.BC_ROOT_DIR = os.getenv("BC_ROOT_DIR", "")
        self.BC_SKIP_MAPPING = convert_str_to_bool(os.getenv("BC_SKIP_MAPPING", False))
        self.BC_SOURCE = os.getenv("BC_SOURCE", "cli")
        self.BC_SOURCE_VERSION = os.getenv("BC_SOURCE_VERSION", version)
        self.CACHE_DIR = convert_str_to_bool(os.getenv("CKV_CACHE_DIR", str(Path(tempfile.gettempdir()) / "cache")))
        self.CHECK_FAIL_LEVEL = os.getenv("CHECKOV_CHECK_FAIL_LEVEL", CheckFailLevel.ERROR)
        self.CREATE_COMPLEX_VERTICES = convert_str_to_bool(os.getenv("CREATE_COMPLEX_VERTICES", True))
        self.CREATE_EDGES = convert_str_to_bool(os.getenv("CREATE_EDGES", True))
        self.CREATE_GRAPH = convert_str_to_bool(os.getenv("CHECKOV_CREATE_GRAPH", True))
        self.CREATE_MARKDOWN_HYPERLINKS = convert_str_to_bool(os.getenv("CHECKOV_CREATE_MARKDOWN_HYPERLINKS", False))
        self.CREATE_SCA_IMAGE_REPORTS_FOR_IR = convert_str_to_bool(
            os.getenv("CHECKOV_CREATE_SCA_IMAGE_REPORTS_FOR_IR", True)
        )
        # default version is set inside the relevant code
        self.CYCLONEDX_SCHEMA_VERSION = os.getenv("CHECKOV_CYCLONEDX_SCHEMA_VERSION", "")
        self.DISPLAY_REGISTRY_URL = convert_str_to_bool(os.getenv("CHECKOV_DISPLAY_REGISTRY_URL", False))
        self.ENABLE_FOREACH_HANDLING = convert_str_to_bool(os.getenv("CHECKOV_ENABLE_FOREACH_HANDLING", True))
        self.ENABLE_MODULES_FOREACH_HANDLING = convert_str_to_bool(
            os.getenv("CHECKOV_ENABLE_MODULES_FOREACH_HANDLING", True)
        )
        self.ENABLE_NESTED_MODULES = convert_str_to_bool(os.getenv("CHECKOV_ENABLE_NESTED_MODULES", True))
        self.EXPERIMENTAL_GRAPH_DEBUG = convert_str_to_bool(os.getenv("CHECKOV_EXPERIMENTAL_GRAPH_DEBUG", False))
        self.EXPIRATION_TIME_IN_SEC = force_int(os.getenv("CHECKOV_EXPIRATION_TIME_IN_SEC", 604800))
        self.GITHUB_CONF_DIR_NAME = os.getenv("CKV_GITHUB_CONF_DIR_NAME", "github_conf")
        self.GITHUB_CONFIG_FETCH_DATA = convert_str_to_bool(os.getenv("CKV_GITHUB_CONFIG_FETCH_DATA", True))
        self.GITLAB_CONF_DIR_NAME = os.getenv("CKV_GITLAB_CONF_DIR_NAME", "gitlab_conf")
        self.GITLAB_CONFIG_FETCH_DATA = convert_str_to_bool(os.getenv("CKV_GITLAB_CONFIG_FETCH_DATA", True))
        self.GRAPH_FRAMEWORK = os.getenv("CHECKOV_GRAPH_FRAMEWORK", "IGRAPH")
        self.IGNORED_DIRECTORIES = os.getenv("CKV_IGNORED_DIRECTORIES", "node_modules,.terraform,.serverless")
        self.IGNORE_HIDDEN_DIRECTORIES = convert_str_to_bool(os.getenv("CKV_IGNORE_HIDDEN_DIRECTORIES", True))
        self.MAX_FILE_SIZE = force_int(os.getenv("CHECKOV_MAX_FILE_SIZE", 5_000_000))  # 5 MB is default limit
        self.MAX_IAC_FILE_SIZE = force_int(os.getenv("CHECKOV_MAX_IAC_FILE_SIZE", 50_000_000))  # 50 MB is default limit
        self.NEW_TF_PARSER = convert_str_to_bool(os.getenv("CHECKOV_NEW_TF_PARSER", True))
        self.NO_OUTPUT = convert_str_to_bool(os.getenv("CHECKOV_NO_OUTPUT", False))
        self.OPENAI_MAX_FINDINGS = force_int(os.getenv("CKV_OPENAI_MAX_FINDINGS", 5))
        self.OPENAI_MAX_TOKENS = force_int(os.getenv("CKV_OPENAI_MAX_TOKENS", 512))
        self.OPENAI_MODEL = os.getenv("CKV_OPENAI_MODEL", "gpt-3.5-turbo")
        self.OUTPUT_CODE_LINE_LIMIT = force_int(os.getenv("CHECKOV_OUTPUT_CODE_LINE_LIMIT", 50))
        self.PARSE_ERROR_FAIL = convert_str_to_bool(os.getenv("CKV_PARSE_ERROR_FAIL", False))
        self.RENDER_ASYNC_MAX_WORKERS = force_int(os.getenv("RENDER_ASYNC_MAX_WORKERS", 50))
        self.RENDER_EDGES_DUPLICATE_ITER_COUNT = force_int(os.getenv("RENDER_EDGES_DUPLICATE_ITER_COUNT", 4))
        self.RENDER_EDGES_DUPLICATE_PERCENT = force_int(os.getenv("RENDER_EDGES_DUPLICATE_PERCENT", 90))
        self.RENDER_MAX_LEN = force_int(os.getenv("CHECKOV_RENDER_MAX_LEN", 10000))
        self.RENDER_VARIABLES_ASYNC = convert_str_to_bool(os.getenv("RENDER_VARIABLES_ASYNC", False))
        self.RUN_IN_DOCKER = convert_str_to_bool(os.getenv("RUN_IN_DOCKER", False))
        self.REQUEST_MAX_TRIES = force_int(os.getenv("REQUEST_MAX_TRIES", 3))
        self.RUN_SCA_PACKAGE_SCAN_V2 = convert_str_to_bool(os.getenv("CHECKOV_RUN_SCA_PACKAGE_SCAN_V2", True))
        self.RUN_SECRETS_MULTIPROCESS = convert_str_to_bool(os.getenv("RUN_SECRETS_MULTIPROCESS", False))
        self.SLEEP_BETWEEN_REQUEST_TRIES = force_int(os.getenv("SLEEP_BETWEEN_REQUEST_TRIES", 1))
        self.SLS_FILE_MASK = os.getenv("CKV_SLS_FILE_MASK", "serverless.yml,serverless.yaml").split(",")
        self.VALIDATE_SECRETS = convert_str_to_bool(os.getenv("CKV_VALIDATE_SECRETS", False))
        self.WORKDIR = os.getenv("WORKDIR", "")

        # possibly not used anymore 'checkov/terraform/graph_builder/utils.generate_possible_strings_from_wildcards()'
        self.MAX_WILDCARD_ARR_SIZE = force_int(os.getenv("MAX_WILDCARD_ARR_SIZE", 10))
        # is also defined as a flag, need to remove env var references in code
        self.EXTERNAL_MODULES_DIR = os.getenv("EXTERNAL_MODULES_DIR", DEFAULT_EXTERNAL_MODULES_DIR)
        # is also defined as a flag, need to remove env var references in code
        self.BC_CA_BUNDLE = os.getenv("BC_CA_BUNDLE")
        # is also defined as a flag, need to remove env var references in code
        self.PRISMA_API_URL = os.getenv("PRISMA_API_URL")
        # need to fix usage, because the env var value is set inside the code
        self.GITHUB_CONF_DIR_PATH = os.getenv("CKV_GITHUB_CONF_DIR_PATH")


env_vars_config = EnvVarsConfig()



from checkov.common.models.enums import CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class EBSEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure all data stored in the EBS is securely encrypted"
        id = "CKV_AWS_3"
        supported_resources = ("AWS::EC2::Volume",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/Encrypted"


check = EBSEncryption()



# -*- coding: utf-8 -*-
"""
======
stopit
======

code from:
https://github.com/glenfant/stopit
package:
https://pypi.org/project/stopit

Public resources from ``stopit``
"""

from .utils import TimeoutException
from .threadstop import ThreadingTimeout, async_raise, threading_timeoutable
from .signalstop import SignalTimeout, signal_timeoutable


__all__ = (
    'ThreadingTimeout', 'async_raise', 'threading_timeoutable',
    'SignalTimeout', 'signal_timeoutable', 'TimeoutException'
)



from __future__ import annotations

import json
import logging
from typing import List, Dict, Optional, Tuple, TYPE_CHECKING, Any

from checkov.cloudformation.cfn_utils import get_folder_definitions
from checkov.cloudformation.context_parser import ContextParser
from checkov.cloudformation.graph_builder.graph_to_definitions import convert_graph_vertices_to_definitions
from checkov.cloudformation.graph_builder.local_graph import CloudformationLocalGraph
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.graph.graph_manager import GraphManager

if TYPE_CHECKING:
    from checkov.common.typing import LibraryGraphConnector


class CloudformationGraphManager(GraphManager[CloudformationLocalGraph, "dict[str, dict[str, Any]]"]):
    def __init__(self, db_connector: LibraryGraphConnector, source: str = GraphSource.CLOUDFORMATION) -> None:
        super().__init__(db_connector=db_connector, parser=None, source=source)

    def build_graph_from_source_directory(
        self,
        source_dir: str,
        local_graph_class: type[CloudformationLocalGraph] = CloudformationLocalGraph,
        render_variables: bool = True,
        parsing_errors: Optional[Dict[str, Exception]] = None,
        download_external_modules: bool = False,
        excluded_paths: Optional[List[str]] = None,
    ) -> Tuple[CloudformationLocalGraph, dict[str, dict[str, Any]]]:
        logging.info(f"[CloudformationGraphManager] Parsing files in source dir {source_dir}")
        parsing_errors = {} if parsing_errors is None else parsing_errors
        definitions, definitions_raw = get_folder_definitions(source_dir, excluded_paths, parsing_errors)  # type:ignore[arg-type]
        local_graph = self.build_graph_from_definitions(definitions, render_variables)
        rendered_definitions, _ = convert_graph_vertices_to_definitions(local_graph.vertices, source_dir)

        # TODO: replace with real graph rendering
        for cf_file in rendered_definitions.keys():
            file_definition = rendered_definitions.get(cf_file, None)
            file_definition_raw = definitions_raw.get(cf_file, None)
            if file_definition is not None and file_definition_raw is not None:
                cf_context_parser = ContextParser(cf_file, file_definition, file_definition_raw)
                logging.debug(
                    f"Template Dump for {cf_file}: {json.dumps(file_definition, indent=2, default=str)}"
                )
                cf_context_parser.evaluate_default_refs()
        return local_graph, rendered_definitions

    def build_graph_from_definitions(
        self, definitions: dict[str, dict[str, Any]], render_variables: bool = True
    ) -> CloudformationLocalGraph:
        local_graph = CloudformationLocalGraph(definitions, source=self.source)
        local_graph.build_graph(render_variables=render_variables)

        return local_graph



from __future__ import annotations

import logging
import os
from pathlib import Path
from typing import Any, List

from checkov.common.bridgecrew.bc_source import IDEsSourceTypes
from checkov.common.sca.commons import should_run_scan
from checkov.common.sca.output import add_to_report_sca_data
from checkov.common.typing import _LicenseStatus
from checkov.common.bridgecrew.platform_integration import bc_integration, FileToPersist
from checkov.common.models.consts import SCANNABLE_PACKAGE_FILES_EXTENSIONS, SCANNABLE_PACKAGE_FILES
from checkov.common.models.enums import ErrorStatus
from checkov.common.output.report import Report
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.base_runner import BaseRunner, ignored_directories
from checkov.runner_filter import RunnerFilter
from checkov.sca_package_2.scanner import Scanner


class Runner(BaseRunner[None]):
    check_type = CheckType.SCA_PACKAGE  # noqa: CCE003  # a static attribute

    def __init__(self, report_type: str = check_type) -> None:
        super().__init__(file_extensions=SCANNABLE_PACKAGE_FILES_EXTENSIONS, file_names=SCANNABLE_PACKAGE_FILES)
        self._check_class: str | None = None
        self._code_repo_path: Path | None = None
        self.report_type = report_type

    def prepare_and_scan(
            self,
            root_folder: str | Path | None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            excluded_file_names: set[str] | None = None,
    ) -> tuple[dict[str, Any] | None, list[FileToPersist]]:
        runner_filter = runner_filter or RunnerFilter()
        excluded_file_names = excluded_file_names or set()

        # skip complete run, if flag '--check' was used without a CVE check ID or the license policies
        if not should_run_scan(runner_filter.checks):
            return None, []

        if not bc_integration.bc_api_key:
            logging.info("The --bc-api-key flag needs to be set to run SCA package scanning")
            return None, []

        if bc_integration.bc_source and bc_integration.bc_source.name in IDEsSourceTypes \
                and not bc_integration.is_prisma_integration():
            logging.info("The --bc-api-key flag needs to be set to a Prisma token for SCA scan for vscode or jetbrains extention")
            return {}, []  # should just return an empty result

        self._code_repo_path = Path(root_folder) if root_folder else None

        if not bc_integration.timestamp and bc_integration.bc_source and not bc_integration.bc_source.upload_results:
            bc_integration.set_s3_integration()

        excluded_paths = {*ignored_directories}
        if runner_filter.excluded_paths:
            excluded_paths.update(runner_filter.excluded_paths)

        uploaded_files: List[FileToPersist] | None = self.upload_package_files(
            root_path=self._code_repo_path,
            files=files,
            excluded_paths=excluded_paths,
            excluded_file_names=excluded_file_names,
        )
        if uploaded_files is None:
            # failure happened during uploading
            return None, []

        if len(uploaded_files) == 0:
            # no packages were uploaded. we can skip the scanning
            return {}, uploaded_files

        scanner = Scanner(self.pbar, root_folder)
        self._check_class = f"{scanner.__module__}.{scanner.__class__.__qualname__}"
        scan_results = scanner.scan()

        if scan_results is not None:
            logging.info(f"SCA package scanning successfully scanned {len(scan_results)} files")

        return scan_results, uploaded_files

    def run(
            self,
            root_folder: str | Path | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)
        scan_results, uploaded_files = self.prepare_and_scan(root_folder, files, runner_filter)
        if scan_results is None:
            report.set_error_status(ErrorStatus.ERROR)
            return report

        for path, result in scan_results.items():
            if not result:
                continue
            bc_integration.source_id = result.get("sourceId")
            package_file_path = Path(path)
            if self._code_repo_path:
                try:
                    package_file_path = package_file_path.relative_to(self._code_repo_path)
                except ValueError:
                    # Path.is_relative_to() was implemented in Python 3.9
                    pass

            vulnerabilities = result.get("vulnerabilities") or []
            packages = result.get("packages") or []

            license_statuses = [_LicenseStatus(package_name=elm["packageName"], package_version=elm["packageVersion"],
                                               policy=elm["policy"], license=elm["license"], status=elm["status"])
                                for elm in result.get("license_statuses") or []]

            rootless_file_path = str(package_file_path).replace(package_file_path.anchor, "", 1)
            inline_suppressions = result.get("inlineSuppressions")

            add_to_report_sca_data(
                report=report,
                check_class=self._check_class,
                scanned_file_path=str(package_file_path),
                rootless_file_path=rootless_file_path,
                runner_filter=runner_filter,
                vulnerabilities=vulnerabilities,
                packages=packages,
                license_statuses=license_statuses,
                report_type=self.report_type,
                dependencies=result.get("dependencies", None),
                inline_suppressions=inline_suppressions,
                used_private_registry=result.get("used_private_reg", False)
            )

        return report

    def upload_package_files(
            self,
            root_path: Path | None,
            files: list[str] | None,
            excluded_paths: set[str],
            excluded_file_names: set[str] | None = None,
    ) -> List[FileToPersist] | None:
        """ upload package files to s3"""
        logging.info("SCA package scanning upload for package files")
        excluded_file_names = excluded_file_names or set()
        package_files_to_persist: List[FileToPersist] = []
        try:
            if root_path:
                for file_path in root_path.glob("**/*"):
                    if (file_path.name in SCANNABLE_PACKAGE_FILES or file_path.suffix in SCANNABLE_PACKAGE_FILES_EXTENSIONS) and not any(
                            p in file_path.parts for p in excluded_paths) and file_path.name not in excluded_file_names:
                        file_path_str = str(file_path)
                        package_files_to_persist.append(
                            FileToPersist(file_path_str, os.path.relpath(file_path_str, root_path)))

            if files:
                root_folder = os.path.split(os.path.commonprefix(files))[0]
                for file in files:
                    file_path = Path(file)
                    if not file_path.exists():
                        logging.warning(f"File {file_path} doesn't exist")
                        continue
                    if file_path.name in SCANNABLE_PACKAGE_FILES or file_path.suffix in SCANNABLE_PACKAGE_FILES_EXTENSIONS:
                        package_files_to_persist.append(FileToPersist(file, os.path.relpath(file, root_folder)))

            logging.info(f"{len(package_files_to_persist)} sca package files found.")
            bc_integration.persist_files(package_files_to_persist)
            return package_files_to_persist
        except Exception:
            logging.debug("Unexpected failure happened during uploading files for package scanning.\n"
                          "the scanning is terminating. details are below.\n"
                          "please try again. if it is repeated, please report.", exc_info=True)
            return None



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class DataExplorerServiceIdentity(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that data explorer/Kusto uses managed identities to access Azure resources securely."
        id = "CKV_AZURE_181"
        supported_resources = ('azurerm_kusto_cluster',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'identity/[0]/type'

    def get_expected_values(self) -> Any:
        return ANY_VALUE


check = DataExplorerServiceIdentity()



import logging
import os
from abc import ABC, abstractmethod
from typing import Optional

from checkov.terraform.module_loading.content import ModuleContent
from checkov.terraform.module_loading.module_params import ModuleParams
from checkov.terraform.module_loading.registry import module_loader_registry


# ModuleContent allows access to a directory containing module file via the `path()`
# function. Instances may be used in a `with` context to ensure temporary directories
# are removed, if applicable.


class ModuleLoader(ABC):
    def __init__(self) -> None:
        module_loader_registry.register(self)
        self.logger = logging.getLogger(__name__)
        self.module_source: str = ""
        self.current_dir: str = ""
        self.dest_dir: str = ""
        self.external_modules_folder_name: str = ""
        self.version = "latest"
        self.is_external = True
        self.inner_module: Optional[str] = None
        self.root_dir = ""  # root dir for storing external modules

    @abstractmethod
    def discover(self, module_params: ModuleParams) -> None:
        """
            discover parameters from execution context of checkov. usually from env variable
        """
        pass

    def load(self, module_params: ModuleParams) -> ModuleContent:
        """
This function provides an opportunity for the loader to load a module's content if it chooses to do so.
There are three resulting states that can occur when calling this function:
 1) the loader can't handle the source type, in which case a ModuleContent is returned for which
    the `loaded()` method will return False.
 2) the loader can handle the source type and loading is successful, in which case a ModuleContent
    object is returned for which `loaded()` returns True and which provides the directory containing
    the module files
 3) the loader tried to load the module content but and error occurred, in which case an exception
    is raised.
        :param module_params: dataclass object that contains all the parameters of the module to load.
                              the data of this object can be changed according to the loader logic
        :return: A ModuleContent object which may or may not being loaded.
        """
        self.discover(module_params)
        if not self._is_matching_loader(module_params):
            return ModuleContent(dir=None)

        module_path = self._find_module_path(module_params)
        if os.path.exists(module_path):
            return ModuleContent(dir=module_path)

        self.logger.debug(f"Using {self.__class__.__name__} attempting to get module "
                          f"{module_params.module_source if '@' not in module_params.module_source else module_params.module_source.split('@')[1]} "
                          f"version: {module_params.version}")
        return self._load_module(module_params)

    @abstractmethod
    def _is_matching_loader(self, module_params: ModuleParams) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def _load_module(self, module_params: ModuleParams) -> ModuleContent:
        raise NotImplementedError()

    @abstractmethod
    def _find_module_path(self, module_params: ModuleParams) -> str:
        raise NotImplementedError()



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class CodeBuildPrivilegedMode(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-2(1), NIST.800-53.r5 AC-3, NIST.800-53.r5 AC-3(15), NIST.800-53.r5 AC-3(7),
        NIST.800-53.r5 AC-5, NIST.800-53.r5 AC-6, NIST.800-53.r5 AC-6(10), NIST.800-53.r5 AC-6(2)
        CodeBuild project environments should not have privileged mode enabled
        """
        name = "Ensure CodeBuild project environments do not have privileged mode enabled"
        id = "CKV_AWS_316"
        supported_resources = ['aws_codebuild_project']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "environment/[0]/privileged_mode"

    def get_forbidden_values(self):
        return [True]


check = CodeBuildPrivilegedMode()



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader

if TYPE_CHECKING:
    from checkov.terraform.module_loading.module_params import ModuleParams


class BitbucketLoader(GenericGitLoader):
    def discover(self, module_params: ModuleParams) -> None:
        self.module_source_prefix = "bitbucket.org"


loader = BitbucketLoader()



from checkov.terraform.checks.resource.alicloud.AbsRDSParameter import AbsRDSParameter


class RDSInstanceLogsEnabled(AbsRDSParameter):
    def __init__(self):
        super().__init__(check_id="CKV_ALI_35", parameter="log_duration")


check = RDSInstanceLogsEnabled()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.models.consts import ANY_VALUE
from typing import Any


class RedshiftClusterDatabaseName(BaseResourceValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-2
        Redshift clusters should not use the default database name
        """
        name = "Ensure Redshift clusters do not use the default database name"
        id = "CKV_AWS_320"
        supported_resources = ('aws_redshift_cluster',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return "database_name"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = RedshiftClusterDatabaseName()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class DefaultNamespace(BaseK8Check):
    def __init__(self) -> None:
        # CIS-1.5 5.7.4
        name = "The default namespace should not be used"
        # default Service Account and Service/kubernetes are ignored
        id = "CKV_K8S_21"
        supported_kind = (
            "Pod",
            "Deployment",
            "DaemonSet",
            "StatefulSet",
            "ReplicaSet",
            "ReplicationController",
            "Job",
            "CronJob",
            "Service",
            "Secret",
            "ServiceAccount",
            "Role",
            "RoleBinding",
            "ConfigMap",
            "Ingress",
        )
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        metadata = conf.get("metadata")
        if metadata:
            if "namespace" in metadata and metadata["namespace"] != "default":
                return CheckResult.PASSED

            # If namespace not defined it is default -> Ignore default Service account and kubernetes service
            if conf["kind"] == "ServiceAccount" and metadata["name"] == "default":
                return CheckResult.PASSED
            if conf["kind"] == "Service" and metadata["name"] == "kubernetes":
                return CheckResult.PASSED
            return CheckResult.FAILED
        return CheckResult.FAILED


check = DefaultNamespace()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class GKEEnableShieldedNodes(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Shielded GKE Nodes are Enabled"
        id = "CKV_GCP_71"
        supported_resources = ('google_container_cluster',)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'enable_shielded_nodes'

    def get_forbidden_values(self) -> list[Any]:
        return [False]


check = GKEEnableShieldedNodes()



from __future__ import annotations

import logging
import os
from typing import TYPE_CHECKING, Any

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.output.graph_record import GraphRecord
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.runners.base_runner import CHECKOV_CREATE_GRAPH
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.secrets import omit_secret_value_from_checks
from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.registry import resource_registry
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.runner import Runner as TerraformRunner
from checkov.terraform_json.utils import get_scannable_file_paths, TF_JSON_POSSIBLE_FILE_ENDINGS, create_definitions

if TYPE_CHECKING:
    from checkov.common.graph.checks_infra.registry import BaseRegistry
    from checkov.terraform.graph_manager import TerraformGraphManager
    from checkov.common.typing import LibraryGraphConnector, _CheckResult

logger = logging.getLogger(__name__)


class TerraformJsonRunner(TerraformRunner):
    check_type = CheckType.TERRAFORM_JSON  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        graph_class: type[TerraformLocalGraph] = TerraformLocalGraph,
        graph_manager: TerraformGraphManager | None = None,
        db_connector: LibraryGraphConnector | None = None,
        external_registries: list[BaseRegistry] | None = None,
        source: str = GraphSource.TERRAFORM,
    ) -> None:
        super().__init__(
            graph_class=graph_class,
            graph_manager=graph_manager,
            db_connector=db_connector,
            external_registries=external_registries,
            source=source,
        )
        self.file_extensions = TF_JSON_POSSIBLE_FILE_ENDINGS  # override what gets set from the TF runner
        self.graph_registry = get_graph_checks_registry(super().check_type)

        self.definitions: dict[str, dict[str, Any]] = {}
        self.context: dict[str, dict[str, Any]] = {}
        self.root_folder: str | None = None

    def run(
        self,
        root_folder: str | None = None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()

        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)
        self.root_folder = root_folder

        if not self.definitions or not self.context:
            file_paths = get_scannable_file_paths(
                root_folder=root_folder, files=files, excluded_paths=runner_filter.excluded_paths
            )

            if not file_paths:
                return report

            self.definitions, self.definitions_raw, parsing_errors = create_definitions(file_paths)

            report.add_parsing_errors(parsing_errors)

            if external_checks_dir:
                for directory in external_checks_dir:
                    resource_registry.load_external_checks(directory)

                    if CHECKOV_CREATE_GRAPH:
                        self.graph_registry.load_external_checks(directory)

            # TODO: create function 'build_definitions_context()'
            # self.context = build_definitions_context(definitions=self.definitions, definitions_raw=self.definitions_raw)

            if CHECKOV_CREATE_GRAPH:
                logger.info("Creating Terraform JSON graph")
                local_graph = self.graph_manager.build_graph_from_definitions(self.definitions)
                logger.info("Successfully created Terraform JSON graph")

                self.graph_manager.save_graph(local_graph)

            self.pbar.initiate(len(self.definitions))

            # run Python checks
            self.add_python_check_results(report=report, runner_filter=runner_filter)

            # run graph checks
            if CHECKOV_CREATE_GRAPH:
                self.add_graph_check_results(report=report, runner_filter=runner_filter)

        return report

    def add_python_check_results(self, report: Report, runner_filter: RunnerFilter) -> None:
        """Adds Python check results to given report"""

        for full_file_path, definition in self.definitions.items():
            scanned_file = f"/{os.path.relpath(full_file_path, self.root_folder)}"
            logger.debug(f"Scanning file: {scanned_file}")
            self.pbar.set_additional_data({"Current File Scanned": scanned_file})
            for block_type in definition.keys():
                if block_type in self.block_type_registries:
                    self.run_block(
                        entities=definition[block_type],
                        definition_context={},
                        full_file_path=full_file_path,
                        root_folder=self.root_folder,
                        report=report,
                        scanned_file=scanned_file,
                        block_type=block_type,
                        runner_filter=runner_filter,
                    )

            self.pbar.update()
        self.pbar.close()

    def add_graph_check_results(self, report: Report, runner_filter: RunnerFilter) -> None:
        """Adds YAML check results to given report"""

        checks_results = self.run_graph_checks_results(
            runner_filter=runner_filter, report_type=CheckType.TERRAFORM_JSON
        )

        for check, check_results in checks_results.items():
            for check_result in check_results:
                entity = check_result["entity"]
                entity_file_path = entity[CustomAttributes.FILE_PATH]

                clean_check_result: "_CheckResult" = {
                    "result": check_result["result"],
                    "evaluated_keys": check_result["evaluated_keys"],
                }

                file_code_lines = self.definitions_raw[entity_file_path]
                start_line = entity[START_LINE]
                end_line = entity[END_LINE]
                scanned_file = f"/{os.path.relpath(entity_file_path, self.root_folder)}"

                record = Record(
                    check_id=check.id,
                    bc_check_id=check.bc_id,
                    check_name=check.name,
                    check_result=clean_check_result,
                    code_block=file_code_lines[start_line - 1 : end_line],
                    file_path=scanned_file,
                    file_line_range=[start_line, end_line],
                    resource=entity[CustomAttributes.ID],
                    check_class=check.__class__.__module__,
                    file_abs_path=str(entity_file_path),
                    evaluations=None,
                    severity=check.severity,
                )
                if self.breadcrumbs:
                    breadcrumb = self.breadcrumbs.get(record.file_path, {}).get(record.resource)
                    if breadcrumb:
                        record = GraphRecord(record, breadcrumb)
                record.set_guideline(guideline=check.guideline)
                report.add_record(record=record)

    def run_block(
        self,
        entities: list[dict[str, Any]],
        definition_context: dict[str, Any],
        full_file_path: str,
        root_folder: str | None,
        report: Report,
        scanned_file: str,
        block_type: str,
        runner_filter: RunnerFilter | None = None,
        entity_context_path_header: list[str] | None = None,
        module_referrer: str | None = None,
    ) -> None:
        """Run block specific checks"""

        runner_filter = runner_filter or RunnerFilter()
        registry = self.block_type_registries[block_type]
        if registry:
            for entity in entities:
                _, entity_name, entity_config = registry.extract_entity_details(entity)

                start_line = entity_config[START_LINE]
                end_line = entity_config[END_LINE]
                entity_id = f"{block_type}.{entity_name}"
                entity_lines_range = [start_line, end_line]
                entity_code_lines = self.definitions_raw[full_file_path][start_line - 1 : end_line]

                results = registry.scan(scanned_file, entity, [], runner_filter, report_type=CheckType.TERRAFORM_JSON)
                for check, check_result in results.items():
                    censored_code_lines = omit_secret_value_from_checks(
                        check=check,
                        check_result=check_result,
                        entity_code_lines=entity_code_lines,
                        entity_config=entity_config,
                        resource_attributes_to_omit=runner_filter.resource_attr_to_omit,
                    )
                    record = Record(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=check_result,
                        code_block=censored_code_lines,
                        file_path=scanned_file,
                        file_line_range=entity_lines_range,
                        resource=entity_id,
                        evaluations=None,
                        check_class=check.__class__.__module__,
                        file_abs_path=full_file_path,
                        severity=check.severity,
                        details=check.details,
                    )
                    record.set_guideline(guideline=check.guideline)
                    report.add_record(record=record)



from checkov.common.vcs.vcs_schema import VCSSchema


class OrgWebhooksSchema(VCSSchema):
    def __init__(self) -> None:
        schema = {
            "type": "array",
            "items": {
                "title": "Org Hook",
                "description": "Org Hook",
                "type": "object",
                "properties": {
                    "id": {
                        "type": "integer",
                        "examples": [
                            1
                        ]
                    },
                    "url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/orgs/octocat/hooks/1"
                        ]
                    },
                    "ping_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/orgs/octocat/hooks/1/pings"
                        ]
                    },
                    "deliveries_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/orgs/octocat/hooks/1/deliveries"
                        ]
                    },
                    "name": {
                        "type": "string",
                        "examples": [
                            "web"
                        ]
                    },
                    "events": {
                        "type": "array",
                        "items": {
                            "type": "string"
                        },
                        "examples": [
                            "push",
                            "pull_request"
                        ]
                    },
                    "active": {
                        "type": "boolean",
                        "examples": [
                            True
                        ]
                    },
                    "config": {
                        "type": "object",
                        "properties": {
                            "url": {
                                "type": "string",
                                "examples": [
                                    "\"http://example.com/2\""
                                ]
                            },
                            "insecure_ssl": {
                                "type": "string",
                                "examples": [
                                    "\"0\""
                                ]
                            },
                            "content_type": {
                                "type": "string",
                                "examples": [
                                    "\"form\""
                                ]
                            },
                            "secret": {
                                "type": "string",
                                "examples": [
                                    "\"********\""
                                ]
                            }
                        }
                    },
                    "updated_at": {
                        "type": "string",
                        "format": "date-time",
                        "examples": [
                            "2011-09-06T20:39:23Z"
                        ]
                    },
                    "created_at": {
                        "type": "string",
                        "format": "date-time",
                        "examples": [
                            "2011-09-06T17:26:27Z"
                        ]
                    },
                    "type": {
                        "type": "string",
                        "const": "Organization"
                    }
                },
                "required": [
                    "id",
                    "url",
                    "type",
                    "name",
                    "active",
                    "events",
                    "config",
                    "ping_url",
                    "created_at",
                    "updated_at"
                ]
            }
        }
        super().__init__(schema=schema)


schema = OrgWebhooksSchema()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ACRUseSignedImages(BaseResourceValueCheck):

    def __init__(self):
        # IN ARM - Set properties.trustPolicy.status to enabled, set
        # properties.trustPolicy.type to Notary
        # This is the default behaviour by the tf provider when the trust policy is enabled
        name = "Ensures that ACR uses signed/trusted images"
        id = "CKV_AZURE_164"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "trust_policy/[0]/enabled"


check = ACRUseSignedImages()



from __future__ import annotations

from typing import Any, TYPE_CHECKING

from checkov.terraform.checks.data.base_cloudsplaining_data_iam_check import BaseTerraformCloudsplainingDataIAMCheck

if TYPE_CHECKING:
    from cloudsplaining.scan.policy_document import PolicyDocument


class ResourcePolicyDocument(BaseTerraformCloudsplainingDataIAMCheck):
    def __init__(self) -> None:
        name = 'Ensure no IAM policies documents allow "*" as a statement\'s resource for restrictable actions'
        id = "CKV_AWS_356"
        super().__init__(name=name, id=id)

    def cloudsplaining_analysis(self, policy: PolicyDocument) -> list[str] | list[dict[str, Any]]:
        return policy.all_allowed_unrestricted_actions


check = ResourcePolicyDocument()



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver


class EqualsAttributeSolver(BaseAttributeSolver):
    operator = Operators.EQUALS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr_val = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None
        if type(attr_val) == bool or type(self.value) == bool:
            # handle cases like str(False) == "false"
            # generally self.value will be a string, but could be a bool if the policy was created straight from json
            return str(attr_val).lower() == str(self.value).lower()
        elif (isinstance(attr_val, list) and isinstance(self.value, list)) or (isinstance(attr_val, dict) and isinstance(self.value, dict)):
            return attr_val == self.value
        else:
            return str(attr_val) == str(self.value)



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class LBTargetGroupDefinesHealthCheck(BaseResourceCheck):
    def __init__(self) -> None:
        """
        PCI v3.2.1
        """

        name = "Ensure HTTP HTTPS Target group defines Healthcheck"
        id = "CKV_AWS_261"
        supported_resources = (
            "aws_lb_target_group",
            "aws_alb_target_group",
        )
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if conf.get("protocol") in (["HTTP"], ["HTTPS"]):
            health_checks = conf.get("health_check")
            if health_checks and isinstance(health_checks, list):
                healthcheck = health_checks[0]
                if isinstance(healthcheck, dict) and healthcheck.get("path"):
                    return CheckResult.PASSED
            self.evaluated_keys = ["health_check"]
            return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = LBTargetGroupDefinesHealthCheck()



from __future__ import annotations

import json
from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class BatchJobIsNotPrivileged(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Batch job does not define a privileged container"
        id = "CKV_AWS_210"
        supported_resources = ("aws_batch_job_definition",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        container_properties = conf.get("container_properties")
        if container_properties:
            if isinstance(container_properties[0], str):
                try:
                    container = json.loads(container_properties[0])
                except json.JSONDecodeError:
                    return CheckResult.UNKNOWN
            else:
                container = container_properties[0]
            if not isinstance(container, dict):
                return CheckResult.UNKNOWN
            if container.get("privileged"):
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = BatchJobIsNotPrivileged()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class CodebuildS3LogsEncrypted(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-3(6), NIST.800-53.r5 SC-13, NIST.800-53.r5 SC-28,
        NIST.800-53.r5 SC-28(1), NIST.800-53.r5 SI-7(6)
        """
        name = "Ensure that CodeBuild S3 logs are encrypted"
        id = "CKV_AWS_311"
        supported_resource = ['aws_codebuild_project']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def get_inspected_key(self):
        return "logs_config/[0]/s3_logs/[0]/encryption_disabled"

    def get_forbidden_values(self):
        return [True]


check = CodebuildS3LogsEncrypted()



from checkov.common.vcs.vcs_schema import VCSSchema


class RepositoryWebhookSchema(VCSSchema):
    def __init__(self) -> None:
        schema = {
            "type": "array",
            "items": {
                "title": "Webhook",
                "description": "Webhooks for repositories.",
                "type": "object",
                "properties": {
                    "type": {
                        "type": "string",
                        "const": "Repository"
                    },
                    "id": {
                        "description": "Unique identifier of the webhook.",
                        "type": "integer",
                        "examples": [
                            42
                        ]
                    },
                    "name": {
                        "description": "The name of a valid service, use 'web' for a webhook.",
                        "type": "string",
                        "examples": [
                            "web"
                        ]
                    },
                    "active": {
                        "description": "Determines whether the hook is actually triggered on pushes.",
                        "type": "boolean",
                        "examples": [
                            True
                        ]
                    },
                    "events": {
                        "description": "Determines what events the hook is triggered for. Default: ['push'].",
                        "type": "array",
                        "items": {
                            "type": "string"
                        },
                        "examples": [
                            "push",
                            "pull_request"
                        ]
                    },
                    "config": {
                        "type": "object",
                        "properties": {
                            "email": {
                                "type": "string",
                                "examples": [
                                    "\"foo@bar.com\""
                                ]
                            },
                            "password": {
                                "type": "string",
                                "examples": [
                                    "\"foo\""
                                ]
                            },
                            "room": {
                                "type": "string",
                                "examples": [
                                    "\"roomer\""
                                ]
                            },
                            "subdomain": {
                                "type": "string",
                                "examples": [
                                    "\"foo\""
                                ]
                            },
                            "url": {
                                "type": "string",
                                "description": "The URL to which the payloads will be delivered.",
                                "format": "uri",
                                "examples": [
                                    "https://example.com/webhook"
                                ]
                            },
                            "insecure_ssl": {
                                "oneOf": [
                                    {
                                        "type": "string",
                                        "description": "Determines whether the SSL certificate of the host for `url` will be verified when delivering payloads. Supported values include `0` (verification is performed) and `1` (verification is not performed). The default is `0`. **We strongly recommend not setting this to `1` as you are subject to man-in-the-middle and other attacks.**",
                                        "examples": [
                                            "\"0\""
                                        ]
                                    },
                                    {
                                        "type": "number"
                                    }
                                ]
                            },
                            "content_type": {
                                "type": "string",
                                "description": "The media type used to serialize the payloads. Supported values include `json` and `form`. The default is `form`.",
                                "examples": [
                                    "\"json\""
                                ]
                            },
                            "digest": {
                                "type": "string",
                                "examples": [
                                    "\"sha256\""
                                ]
                            },
                            "secret": {
                                "type": "string",
                                "description": "If provided, the `secret` will be used as the `key` to generate the HMAC hex digest value for [delivery signature headers](https://docs.github.com/webhooks/event-payloads/#delivery-headers).",
                                "examples": [
                                    "\"********\""
                                ]
                            },
                            "token": {
                                "type": "string",
                                "examples": [
                                    "\"abc\""
                                ]
                            }
                        }
                    },
                    "updated_at": {
                        "type": "string",
                        "format": "date-time",
                        "examples": [
                            "2011-09-06T20:39:23Z"
                        ]
                    },
                    "created_at": {
                        "type": "string",
                        "format": "date-time",
                        "examples": [
                            "2011-09-06T17:26:27Z"
                        ]
                    },
                    "url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/repos/octocat/Hello-World/hooks/1"
                        ]
                    },
                    "test_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/repos/octocat/Hello-World/hooks/1/test"
                        ]
                    },
                    "ping_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/repos/octocat/Hello-World/hooks/1/pings"
                        ]
                    },
                    "deliveries_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/repos/octocat/Hello-World/hooks/1/deliveries"
                        ]
                    },
                    "last_response": {
                        "title": "Hook Response",
                        "type": "object",
                        "properties": {
                            "code": {
                                "type": [
                                    "integer",
                                    "null"
                                ]
                            },
                            "status": {
                                "type": [
                                    "string",
                                    "null"
                                ]
                            },
                            "message": {
                                "type": [
                                    "string",
                                    "null"
                                ]
                            }
                        },
                        "required": [
                            "code",
                            "status",
                            "message"
                        ]
                    }
                },
                "required": [
                    "id",
                    "url",
                    "type",
                    "name",
                    "active",
                    "events",
                    "config",
                    "ping_url",
                    "created_at",
                    "updated_at",
                    "last_response",
                    "test_url"
                ]
            }
        }
        super().__init__(schema=schema)


schema = RepositoryWebhookSchema()



import abc

from checkov.common.output.record import Record
from checkov.policies_3d.syntax.syntax import Predicate


class SecretsPredicate(Predicate):
    def __init__(self, record: Record) -> None:
        super().__init__()
        self.record = record

    @abc.abstractmethod
    def __call__(self) -> bool:
        raise NotImplementedError()



from __future__ import annotations

import os
import re
from collections.abc import Iterable

from checkov.common.runners.base_runner import ignored_directories, safe_remove
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR

EXCLUDED_PATHS = [*ignored_directories, DEFAULT_EXTERNAL_MODULES_DIR, ".idea", ".git", "venv"]


def filter_excluded_paths(
    root_dir: str,
    names: list[str] | list[os.DirEntry[str]],
    excluded_paths: Iterable[str] | None,
) -> None:
    """Special build of checkov.common.runners.base_runner.filter_ignored_paths for Secrets scanning"""

    # support for the --skip-path flag
    if excluded_paths:
        compiled = []
        for p in excluded_paths:
            try:
                compiled.append(re.compile(p.replace(".terraform", r"\.terraform")))
            except re.error:
                # do not add compiled paths that aren't regexes
                continue
        for entry in list(names):
            path = entry.name if isinstance(entry, os.DirEntry) else entry
            full_path = os.path.join(root_dir, path)
            if any(pattern.search(full_path) for pattern in compiled) or any(p in full_path for p in excluded_paths):
                safe_remove(names, entry)

    # support for our own excluded paths list
    for entry in list(names):
        path = entry.name if isinstance(entry, os.DirEntry) else entry
        if path in EXCLUDED_PATHS:
            safe_remove(names, entry)



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CloudfrontDistributionOriginFailover(BaseResourceCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 CP-10, NIST.800-53.r5 SC-36, NIST.800-53.r5 SC-5(2), NIST.800-53.r5 SI-13(5)
        CloudFront distributions should have origin failover configured
        """
        name = "Ensure CloudFront distributions should have origin failover configured"
        id = "CKV_AWS_310"
        supported_resources = ('aws_cloudfront_distribution',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        groups = conf.get("origin_group")
        if groups and isinstance(groups, list):
            for group in groups:
                if isinstance(group, dict) and group.get("failover_criteria"):
                    member = group.get("member")
                    if not member or len(member) < 2:
                        return CheckResult.FAILED
                else:
                    return CheckResult.FAILED
        else:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = CloudfrontDistributionOriginFailover()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
from checkov.common.util.type_forcers import force_int


class AbsSecurityGroupUnrestrictedIngress(BaseResourceCheck):
    def __init__(self, check_id, port):
        name = f"Ensure no security groups allow ingress from 0.0.0.0:0 to port {port:d} (tcp / udp)"
        supported_resources = ['openstack_compute_secgroup_v2', 'openstack_networking_secgroup_rule_v2']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def scan_resource_conf(self, conf):
        """
            Looks for configuration at security group ingress rules:
            https://registry.terraform.io/providers/terraform-provider-openstack/openstack/latest/docs/resources/compute_secgroup_v2
            https://registry.terraform.io/providers/terraform-provider-openstack/openstack/latest/docs/resources/networking_secgroup_rule_v2

            Return PASS if:
            - the resource is an openstack_compute_secgroup_v2 that contains no violating rules (including if there are
              no rules at all), OR
            - the resource is an openstack_networking_secgroup_rule_v2 of type 'ingress' that does not violate the
              check.

            Return FAIL if:
            - the resource is an openstack_compute_secgroup_v2 that contains a violating rule, OR
            - the resource is an openstack_networking_secgroup_rule_v2 of type 'ingress' that violates the check.

            Return UNKNOWN if:
            - the resource is an openstack_networking_secgroup_rule_v2 of type 'egress'

        :param conf: openstack_compute_secgroup_v2|openstack_networking_secgroup_rule_v2 configuration
        :return: <CheckResult>
        """

        if 'rule' in conf:  # This means it's an SG resource with rule block(s)
            rules = conf['rule']
            for rule in rules:
                if isinstance(rule, dict) and self.contains_violation(rule, 'ip_protocol', 'from_port', 'to_port',
                                                                      'cidr'):
                    self.evaluated_keys = [
                        f'rule/[{rules.index(rule)}]/ip_protocol',
                        f'rule/[{rules.index(rule)}]/from_port',
                        f'rule/[{rules.index(rule)}]/to_port',
                        f'rule/[{rules.index(rule)}]/cidr',
                    ]
                    return CheckResult.FAILED

            return CheckResult.PASSED

        if 'direction' in conf:  # This means it's an SG_rule resource.
            direction = force_list(conf['direction'])[0]
            if direction == 'ingress':
                self.evaluated_keys = ['protocol', 'port_range_min', 'port_range_max', 'remote_ip_prefix']
                if self.contains_violation(conf, 'protocol', 'port_range_min', 'port_range_max', 'remote_ip_prefix'):
                    return CheckResult.FAILED
                return CheckResult.PASSED
            return CheckResult.UNKNOWN

        # The result for an SG with no ingress block
        return CheckResult.PASSED

    def contains_violation(self, conf, protocol_key, from_port_key, to_port_key, cidr_key):
        protocol = force_list(conf.get(protocol_key, [{-1}]))[0]
        from_port = force_int(force_list(conf.get(from_port_key, [{-1}]))[0])
        to_port = force_int(force_list(conf.get(to_port_key, [{-1}]))[0])

        if protocol == "icmp":
            return False

        if from_port is not None and to_port is not None and (from_port <= self.port <= to_port):
            cidr = conf.get(cidr_key, [])
            if len(cidr) > 0 and cidr[0] in ['0.0.0.0/0', '::/0', '0000:0000:0000:0000:0000:0000:0000:0000/0']:
                return True
        return False



from checkov.terraform.checks.resource.azure.NSGRulePortAccessRestricted import NSGRulePortAccessRestricted


class NSGRuleHTTPAccessRestricted(NSGRulePortAccessRestricted):
    def __init__(self) -> None:
        super().__init__(
            name="Ensure that HTTP (port 80) access is restricted from the internet",
            check_id="CKV_AZURE_160",
            port=80,
        )


check = NSGRuleHTTPAccessRestricted()



from __future__ import annotations

import itertools
import json
from typing import TYPE_CHECKING, Any

from checkov.common.models.enums import CheckResult
from checkov.common.output.cyclonedx_consts import SCA_CHECKTYPES
from checkov.common.util.http_utils import valid_url
from checkov.version import version

if TYPE_CHECKING:
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report

SEVERITY_TO_SARIF_LEVEL = {
    "critical": "error",
    "high": "error",
    "medium": "warning",
    "low": "note",
    "none": "none",
}


SEVERITY_TO_SCORE = {
    "critical": "10.0",
    "high": "8.9",
    "medium": "6.9",
    "low": "3.9",
    "none": "0.0",
}


class Sarif:
    def __init__(self, reports: list[Report], tool: str | None) -> None:
        self.reports = reports
        self.rule_index_map: "dict[str, int]" = {}
        self.tool = tool if tool else "Bridgecrew"

        self.json = self.create_json()

    def create_json(self) -> dict[str, Any]:
        return {
            "$schema": "https://raw.githubusercontent.com/oasis-tcs/sarif-spec/master/Schemata/sarif-schema-2.1.0.json",
            "version": "2.1.0",
            "runs": self._create_runs(),
        }

    def _create_runs(self) -> list[dict[str, Any]]:
        information_uri = "https://docs.bridgecrew.io" if self.tool.lower() == "bridgecrew" else "https://checkov.io"
        rules = self._create_rules()  # needs to be invoked before _create_results()
        results = self._create_results()

        return [
            {
                "tool": {
                    "driver": {
                        "name": self.tool,
                        "version": version,
                        "informationUri": information_uri,
                        "rules": rules,
                        "organization": "bridgecrew",
                    }
                },
                "results": results,
            }
        ]

    def _create_rules(self) -> list[dict[str, Any]]:
        rule_idx = 0
        rules: "list[dict[str, Any]]" = []

        for report in self.reports:
            if report.check_type in SCA_CHECKTYPES:
                for record in itertools.chain(report.failed_checks, report.skipped_checks):
                    rule = None
                    if record.check_id.startswith("BC_LIC"):
                        rule = self._create_license_rule(check_type=report.check_type, record=record)
                    elif record.check_id.startswith(("BC_VUL", "CKV_CVE")):
                        rule = self._create_cve_rule(check_type=report.check_type, record=record)

                    if rule and rule["id"] not in self.rule_index_map:
                        self.rule_index_map[rule["id"]] = rule_idx
                        rules.append(rule)
                        rule_idx += 1
            else:
                for record in itertools.chain(report.failed_checks, report.skipped_checks):
                    if record.check_id not in self.rule_index_map:
                        rule = self._create_iac_rule(check_type=report.check_type, record=record)
                        self.rule_index_map[rule["id"]] = rule_idx
                        rules.append(rule)
                        rule_idx += 1

        return rules

    def _create_iac_rule(self, check_type: str, record: Record) -> dict[str, Any]:
        rule = {
            "id": self._create_rule_id(check_type=check_type, record=record),
            "name": record.short_description or record.check_name,
            "shortDescription": {
                "text": record.short_description or record.check_name,
            },
            "fullDescription": {
                "text": record.description or record.check_name,
            },
            "help": {
                "text": f"{record.check_name}\nResource: {record.resource}",
            },
            "defaultConfiguration": {"level": "error"},
        }

        # Adding 'properties' dictionary only if 'record.severity' exists
        if record.severity:
            rule["properties"] = {
                "security-severity": SEVERITY_TO_SCORE.get(record.severity.name.lower(), "0.0"),
            }

        help_uri = record.guideline
        if valid_url(help_uri):
            rule["helpUri"] = help_uri

        return rule

    def _create_cve_rule(self, check_type: str, record: Record) -> dict[str, Any] | None:
        details = record.vulnerability_details
        if not details:
            # this shouldn't happen
            return None

        rule = {
            "id": self._create_rule_id(check_type=check_type, record=record),
            "name": record.short_description or record.check_name,
            "shortDescription": {
                "text": record.short_description or record.check_name,
            },
            "fullDescription": {
                "text": record.description or record.check_name,
            },
            "help": {
                "text": f"{record.check_name}\nResource: {record.resource}\nStatus: {details.get('status')}",
            },
            "defaultConfiguration": {"level": "error"},
        }

        # Add properties dictionary with security-severity
        cvss = details.get("cvss")
        if cvss:
            # use CVSS, if exists
            rule["properties"] = {
                "security-severity": str(cvss),
            }
        elif record.severity:
            # otherwise severity, if exists
            rule["properties"] = {
                "security-severity": SEVERITY_TO_SCORE.get(record.severity.name.lower(), "0.0"),
            }

        help_uri = details.get("link")
        if valid_url(help_uri):
            rule["helpUri"] = help_uri

        return rule

    def _create_license_rule(self, check_type: str, record: Record) -> dict[str, Any] | None:
        details = record.vulnerability_details
        if not details:
            # this shouldn't happen
            return None

        rule = {
            "id": self._create_rule_id(check_type=check_type, record=record),
            "name": record.short_description or record.check_name,
            "shortDescription": {
                "text": record.short_description or record.check_name,
            },
            "fullDescription": {
                "text": f"Package {details['package_name']}@{details['package_version']} has license {details['license']}",
            },
            "help": {
                "text": f"{record.check_name}\nResource: {record.resource}",
            },
            "defaultConfiguration": {"level": "error"},
        }

        # Adding 'properties' dictionary only if 'record.severity' exists
        if record.severity:
            rule["properties"] = {
                "security-severity": SEVERITY_TO_SCORE.get(record.severity.name.lower(), "0.0"),
            }

        help_uri = record.guideline
        if valid_url(help_uri):
            rule["helpUri"] = help_uri

        return rule

    def _create_results(self) -> list[dict[str, Any]]:
        results: "list[dict[str, Any]]" = []

        for report in self.reports:
            for record in itertools.chain(report.failed_checks, report.skipped_checks):
                level = "warning"
                if record.severity:
                    level = SEVERITY_TO_SARIF_LEVEL.get(record.severity.name.lower(), "none")
                elif record.check_result.get("result") == CheckResult.FAILED:
                    level = "error"

                rule_id = self._create_rule_id(check_type=report.check_type, record=record)
                if not rule_id or rule_id not in self.rule_index_map:
                    # can happen if data is missing
                    continue

                result = {
                    "ruleId": rule_id,
                    "ruleIndex": self.rule_index_map[rule_id],
                    "level": level,
                    "attachments": [{"description": detail} for detail in record.details],
                    "message": {
                        "text": record.short_description or record.check_name,
                    },
                    "locations": [
                        {
                            "physicalLocation": {
                                "artifactLocation": {"uri": record.repo_file_path.lstrip("/")},
                                "region": {
                                    "startLine": int(record.file_line_range[0]) or 1,
                                    "endLine": int(record.file_line_range[1]) or 1,
                                    "snippet": {"text": "".join(line for _, line in record.code_block)},
                                },
                            }
                        }
                    ],
                }

                if record.check_result.get("result") == CheckResult.SKIPPED:
                    # sca_package suppression can only be enabled via flag
                    # other runners only report in source suppression
                    kind = "external" if record.vulnerability_details else "inSource"
                    justification = record.check_result.get("suppress_comment")
                    if justification is None:
                        justification = "No comment provided"

                    result["suppressions"] = [
                        {
                            "kind": kind,
                            "justification": justification,
                        }
                    ]

                results.append(result)

        return results

    def _create_rule_id(self, check_type: str, record: Record) -> str | None:
        if check_type in SCA_CHECKTYPES:
            details = record.vulnerability_details
            if not details:
                # this shouldn't happen
                return None

            if record.check_id.startswith("BC_LIC"):
                return f"{details['license']}_{details['package_name']}@{details['package_version']}".replace(" ", "_")
            elif record.check_id.startswith(("BC_VUL", "CKV_CVE")):
                return f"{details['id']}_{details['package_name']}@{details['package_version']}".replace(" ", "_")
        else:
            return record.check_id

        return None

    def write_sarif_output(self) -> None:
        try:
            with open("results.sarif", "w") as f:
                f.write(json.dumps(self.json))
                print("\nWrote output in SARIF format to the file 'results.sarif'")
        except EnvironmentError as e:
            print("\nAn error occurred while writing SARIF results to file: results.sarif")
            print(f"More details: \n {e}")



from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar

import networkx as nx

from checkov.common.graph.db_connectors.db_connector import DBConnector
from checkov.common.graph.graph_builder import CustomAttributes

if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.graph_components.blocks import Block  # noqa
    from checkov.common.graph.graph_builder.local_graph import LocalGraph

_Block = TypeVar("_Block", bound="Block")


class NetworkxConnector(DBConnector[nx.DiGraph]):
    def __init__(self) -> None:
        self.graph = nx.DiGraph()

    def save_graph(self, local_graph: LocalGraph[_Block], add_bulk_edges: bool = False) -> nx.DiGraph:
        return self.networkx_from_local_graph(local_graph)

    def get_reader_endpoint(self) -> nx.DiGraph:
        return self.graph

    def get_writer_endpoint(self) -> nx.DiGraph:
        return self.graph

    def networkx_from_local_graph(self, local_graph: LocalGraph[_Block]) -> nx.DiGraph:
        self.graph = nx.DiGraph()
        vertices_attributes = [v.get_attribute_dict() for v in local_graph.vertices]
        vertices_to_add = [(attr[CustomAttributes.HASH], attr) for attr in vertices_attributes]
        edges_to_add = [
            (
                vertices_attributes[e.origin][CustomAttributes.HASH],
                vertices_attributes[e.dest][CustomAttributes.HASH],
                {"label": e.label},
            )
            for e in local_graph.edges
        ]

        self.graph.add_nodes_from(vertices_to_add)
        self.graph.add_edges_from(edges_to_add)

        return self.graph

    def disconnect(self) -> None:
        # not used, but is an abstractmethod
        return None



from __future__ import annotations

import logging
import os
import re
from typing import List, Callable

from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.terraform.module_loading.registry import module_loader_registry


class ModuleDownload:
    def __init__(self, source_dir: str) -> None:
        self.source_dir = source_dir
        self.module_link: str | None = None
        self.version: str | None = None

    def __str__(self) -> str:
        return f"{self.source_dir} -> {self.module_link} ({self.version})"

    @property
    def address(self) -> str:
        return f'{self.module_link}:{self.version}'


def find_modules(path: str) -> List[ModuleDownload]:
    modules_found = []
    for root, _, full_file_names in os.walk(path):
        for file_name in full_file_names:
            if not file_name.endswith('.tf'):
                continue
            with open(os.path.join(path, root, file_name)) as f:
                try:
                    in_module = False
                    curr_md = None
                    for line in f:
                        if line.strip().startswith('#'):
                            continue
                        if not in_module:
                            if line.startswith('module'):
                                in_module = True
                                curr_md = ModuleDownload(os.path.dirname(os.path.join(root, file_name)))
                                continue
                        if in_module and curr_md:
                            if line.startswith('}'):
                                in_module = False
                                if curr_md.module_link is None:
                                    logging.warning(f'A module at {curr_md.source_dir} had no source, skipping')
                                else:
                                    modules_found.append(curr_md)
                                curr_md = None
                                continue

                            match = re.match(re.compile('.*\\bsource\\s*=\\s*"(?P<LINK>.*)"'), line)
                            if match:
                                curr_md.module_link = match.group('LINK')
                                continue

                            match = re.match(re.compile('.*\\bversion\\s*=\\s*"(?P<operator>=|!=|>=|>|<=|<|~>)?\\s*(?P<version>[\\d.]+-?\\w*)"'), line)
                            if match:
                                curr_md.version = f"{match.group('operator')}{match.group('version')}" if match.group('operator') else match.group('version')
                except (UnicodeDecodeError, FileNotFoundError) as e:
                    logging.warning(f"Skipping {os.path.join(path, root, file_name)} because of {e}")
                    continue

    return modules_found


def should_download(path: str | None) -> bool:

    return path is not None and not (path.startswith('./') or path.startswith('../') or path.startswith('/'))


def load_tf_modules(
    path: str,
    should_download_module: Callable[[str | None], bool] = should_download,
    run_parallel: bool = False,
    modules_to_load: List[ModuleDownload] | None = None
) -> None:
    module_loader_registry.root_dir = path
    if not modules_to_load:
        modules_to_load = find_modules(path)

    def _download_module(m: ModuleDownload) -> None:
        if should_download_module(m.module_link):
            logging.info(f'Downloading module {m.address}')
            try:
                content = module_loader_registry.load(m.source_dir, m.module_link,
                                                      "latest" if not m.version else m.version)
                if content is None or not content.loaded():
                    log_message = f'Failed to download module {m.address}'
                    if not module_loader_registry.download_external_modules:
                        log_message += ' (for external modules, the --download-external-modules flag is required)'
                    logging.warning(log_message)
            except Exception as e:
                logging.warning(f"Unable to load module ({m.address}): {e}")

    # To avoid duplicate work, we need to get the distinct module sources
    distinct_modules = list({m.address: m for m in modules_to_load}.values())

    if run_parallel:
        list(parallel_runner.run_function(_download_module, distinct_modules))
    else:
        for m in distinct_modules:
            _download_module(m)



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.gcp.AbsGooglePostgresqlDatabaseFlags import AbsGooglePostgresqlDatabaseFlags

FLAG_NAME = 'log_min_error_statement'
FLAG_VALUES = [
    "debug5",
    "debug4",
    "debug3",
    "debug2",
    "debug1",
    "info",
    "notice",
    "warning",
    "error"
]


class GoogleCloudPostgreSqlLogMinErrorStatement(AbsGooglePostgresqlDatabaseFlags):
    def __init__(self):
        name = "Ensure the GCP PostgreSQL database log levels are set to ERROR or lower"
        check_id = "CKV_GCP_109"
        supported_resources = ['google_sql_database_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(
            name=name,
            id=check_id,
            categories=categories,
            supported_resources=supported_resources,
            flag_name=FLAG_NAME,
            flag_values=FLAG_VALUES
        )


check = GoogleCloudPostgreSqlLogMinErrorStatement()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class K8sEnableNetworkPolicies(BaseResourceCheck):
    def __init__(self):
        """
        Kubernetes Cluster should have Terway or Flannel as CNI Network Plugin as it allows you to use
        standard Kubernetes network policies
        https://www.alibabacloud.com/help/en/container-service-for-kubernetes/latest/work-with-terway
        https://registry.terraform.io/providers/aliyun/alicloud/latest/docs/resources/cs_kubernetes#cluster_network_type
        The vswitches for the pod network when using Terway.Be careful the pod_vswitch_ids can not equal to
        worker_vswitch_ids or master_vswitch_ids but must be in same availability zones.
        Flannel requires pod_cidr.
        """

        name = "Ensure Kubernetes installs plugin Terway or Flannel to support standard policies"
        id = "CKV_ALI_26"
        supported_resources = ['alicloud_cs_kubernetes']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        # required fields
        if not (conf.get("pod_vswitch_ids") or conf.get("pod_cidr")):
            return CheckResult.FAILED
        # addons
        if conf.get("addons") and isinstance(conf.get("addons"), list):
            names = [
                addon["name"][0]
                for addon in conf["addons"]
                if addon.get("name")
            ]

            if "terway-eniip" in names:
                if conf.get("pod_vswitch_ids") and isinstance(conf.get("pod_vswitch_ids"), list):
                    net_ids = conf.get("pod_vswitch_ids")[0]
                    if isinstance(conf.get("worker_vswitch_ids"), list) \
                            and isinstance(conf.get("master_vswitch_ids"), list):
                        if any(net_id in net_ids for net_id in conf.get("worker_vswitch_ids")[0]):
                            self.evaluated_keys = ["worker_vswitch_ids"]
                            return CheckResult.FAILED
                        if any(net_id in net_ids for net_id in conf.get("master_vswitch_ids")[0]):
                            self.evaluated_keys = ["master_vswitch_ids"]
                            return CheckResult.FAILED
                        return CheckResult.PASSED

                self.evaluated_keys = ["pod_vswitch_ids"]
                return CheckResult.FAILED
            if "flannel" in names:
                if conf.get("pod_cidr"):
                    return CheckResult.PASSED
            self.evaluated_keys = ["addons/[0]/config"]
            return CheckResult.FAILED
        else:
            self.evaluated_keys = ["addons"]
            return CheckResult.FAILED


check = K8sEnableNetworkPolicies()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class MemoryRequests(BaseResourceCheck):
    def __init__(self):
        name = "Memory requests should be set"
        id = "CKV_K8S_13"
        supported_resources = ["kubernetes_pod", "kubernetes_pod_v1",
                               "kubernetes_deployment", "kubernetes_deployment_v1"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if "spec" not in conf:
            self.evaluated_keys = [""]
            return CheckResult.FAILED
        spec = conf['spec'][0]
        evaluated_keys_path = "spec"
        if not spec:
            return CheckResult.UNKNOWN

        template = spec.get("template")
        if template and isinstance(template, list):
            template = template[0]
            template_spec = template.get("spec")
            if template_spec and isinstance(template_spec, list):
                spec = template_spec[0]
                evaluated_keys_path = f'{evaluated_keys_path}/[0]/template/[0]/spec'

        containers = spec.get("container")
        if containers is None:
            return CheckResult.UNKNOWN
        for idx, container in enumerate(containers):
            if type(container) != dict:
                return CheckResult.UNKNOWN
            if container.get("resources"):
                resources = container.get("resources")[0]
                if resources.get('requests'):
                    requests = resources.get('requests')[0]
                    if isinstance(requests, dict) and requests.get('memory'):  # nosec  # false positive
                        return CheckResult.PASSED
                    self.evaluated_keys = [f'{evaluated_keys_path}/[0]/container/[{idx}]/resources/[0]/requests']
                    return CheckResult.FAILED
                self.evaluated_keys = [f'{evaluated_keys_path}/[0]/container/[{idx}]/resources']
                return CheckResult.FAILED
            self.evaluated_keys = [f'{evaluated_keys_path}/[0]/container/[{idx}]']
            return CheckResult.FAILED
        return CheckResult.PASSED


check = MemoryRequests()



from __future__ import annotations

import abc
import json
import re
import typing
from typing import Any

from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.terraform import TFModule
from checkov.terraform.graph_builder.foreach.consts import COUNT_STRING, FOREACH_STRING, COUNT_KEY, EACH_VALUE, \
    EACH_KEY, REFERENCES_VALUES
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.variable_rendering.evaluate_terraform import evaluate_terraform
from checkov.terraform.graph_builder.variable_rendering.renderer import TerraformVariableRenderer

if typing.TYPE_CHECKING:
    from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph


class ForeachAbstractHandler:
    def __init__(self, local_graph: TerraformLocalGraph) -> None:
        self.local_graph = local_graph

    @abc.abstractmethod
    def handle(self, resources_blocks: list[int]) -> None:
        pass

    @abc.abstractmethod
    def _create_new_foreach_resource(self, block_idx: int, foreach_idx: int, main_resource: TerraformBlock,
                                     new_key: int | str, new_value: int | str) -> None:
        pass

    @abc.abstractmethod
    def _create_new_resources_count(self, statement: int, block_idx: int) -> None:
        pass

    def _create_new_resources_foreach(self, statement: list[str] | dict[str, Any], block_idx: int) -> None:
        main_resource = self.local_graph.vertices[block_idx]
        if isinstance(statement, list):
            for i, new_value in enumerate(statement):
                self._create_new_foreach_resource(block_idx, i, main_resource, new_key=new_value, new_value=new_value)
        if isinstance(statement, dict):
            for i, (new_key, new_value) in enumerate(statement.items()):
                self._create_new_foreach_resource(block_idx, i, main_resource, new_key, new_value)

    @staticmethod
    def _render_sub_graph(sub_graph: TerraformLocalGraph, blocks_to_render: list[int]) -> None:
        renderer = TerraformVariableRenderer(sub_graph)
        renderer.vertices_index_to_render = blocks_to_render
        renderer.render_variables_from_local_graph()

    def _build_sub_graph(self, blocks_to_render: list[int]) -> TerraformLocalGraph:
        from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph

        sub_graph = TerraformLocalGraph(self.local_graph.module)
        sub_graph.vertices = [{}] * len(self.local_graph.vertices)
        for i, block in enumerate(self.local_graph.vertices):
            if not (block.block_type == BlockType.RESOURCE and i not in blocks_to_render):
                sub_graph.vertices[i] = pickle_deepcopy(block)
        sub_graph.edges = [
            edge for edge in self.local_graph.edges if
            (sub_graph.vertices[edge.dest] and sub_graph.vertices[edge.origin])
        ]
        sub_graph.in_edges = self.local_graph.in_edges
        sub_graph.out_edges = self.local_graph.out_edges
        return sub_graph

    @staticmethod
    def _update_nested_tf_module_foreach_idx(original_foreach_or_count_key: int | str, original_module_key: TFModule,
                                             tf_moudle: TFModule | None) -> None:
        original_module_key.foreach_idx = None  # Make sure it is always None even if we didn't override it previously
        while tf_moudle is not None:
            if tf_moudle == original_module_key:
                tf_moudle.foreach_idx = original_foreach_or_count_key
                break
            tf_moudle = tf_moudle.nested_tf_module

    @staticmethod
    def _pop_foreach_attrs(attrs: dict[str, Any]) -> None:
        attrs.pop(COUNT_STRING, None)
        attrs.pop(FOREACH_STRING, None)

    @staticmethod
    def __update_str_attrs(attrs: dict[str, Any], key_to_change: str, val_to_change: str | dict[str, Any], k: str) -> bool:
        if key_to_change not in attrs[k]:
            return False
        if attrs[k] == "${" + key_to_change + "}":
            attrs[k] = val_to_change
            return True
        elif f"{key_to_change}." in attrs[k] and isinstance(val_to_change, dict):
            key = attrs[k].replace("}", "").split('.')[-1]
            attrs[k] = val_to_change.get(key)
            return True
        else:
            attrs[k] = attrs[k].replace("${" + key_to_change + "}", str(val_to_change))
            attrs[k] = attrs[k].replace(key_to_change, str(val_to_change))
            return True

    @staticmethod
    def _build_key_to_val_changes(main_resource: TerraformBlock, new_val: str | int, new_key: str | int | None)\
            -> dict[str, str | int | None]:
        if main_resource.attributes.get(COUNT_STRING):
            return {COUNT_KEY: new_val}

        return {
            EACH_VALUE: new_val,
            EACH_KEY: new_key
        }

    def _update_foreach_attrs(self, config_attrs: dict[str, Any], key_to_val_changes: dict[str, Any],
                              new_resource: TerraformBlock) -> None:
        self._pop_foreach_attrs(new_resource.attributes)
        self._pop_foreach_attrs(config_attrs)
        self._update_attributes(new_resource.attributes, key_to_val_changes)
        foreach_attrs = self._update_attributes(config_attrs, key_to_val_changes)
        new_resource.foreach_attrs = foreach_attrs

    def _update_attributes(self, attrs: dict[str, Any], key_to_val_changes: dict[str, Any]) -> list[str]:
        foreach_attributes: list[str] = []
        for key_to_change, val_to_change in key_to_val_changes.items():
            for k, v in attrs.items():
                v_changed = False
                if isinstance(v, str):
                    v_changed = self.__update_str_attrs(attrs, key_to_change, val_to_change, k)
                elif isinstance(v, dict):
                    nested_attrs = self._update_attributes(v, {key_to_change: val_to_change})
                    foreach_attributes.extend([k + '.' + na for na in nested_attrs])
                elif isinstance(v, list) and len(v) == 1 and isinstance(v[0], dict):
                    nested_attrs = self._update_attributes(v[0], {key_to_change: val_to_change})
                    foreach_attributes.extend([k + '.' + na for na in nested_attrs])
                elif isinstance(v, list) and len(v) == 1 and isinstance(v[0], str) and key_to_change in v[0]:
                    if attrs[k][0] == "${" + key_to_change + "}":
                        attrs[k][0] = val_to_change
                        v_changed = True
                    elif f"{key_to_change}." in attrs[k][0] and isinstance(val_to_change, dict):
                        key = attrs[k][0].replace("}", "").split('.')[-1]
                        attrs[k][0] = val_to_change.get(key)
                        v_changed = True
                    else:
                        attrs[k][0] = attrs[k][0].replace("${" + key_to_change + "}", str(val_to_change))
                        if self.need_to_add_quotes(attrs[k][0], key_to_change):
                            attrs[k][0] = attrs[k][0].replace(key_to_change, f'"{str(val_to_change)}"')
                        else:
                            attrs[k][0] = attrs[k][0].replace(key_to_change, str(val_to_change))
                        v_changed = True
                elif isinstance(v, list) and len(v) == 1 and isinstance(v[0], list):
                    for i, item in enumerate(v):
                        if isinstance(item, str) and (key_to_change in item or "${" + key_to_change + "}" in item):
                            if v[i] == "${" + key_to_change + "}":
                                v[i] = val_to_change
                                v_changed = True
                            else:
                                v[i] = item.replace("${" + key_to_change + "}", str(val_to_change))
                                v[i] = v[i].replace(key_to_change, str(val_to_change))
                                v_changed = True
                if v_changed:
                    foreach_attributes.append(k)
        return foreach_attributes

    @staticmethod
    def _update_block_name_and_id(block: TerraformBlock, idx: int | str) -> str:
        # Note it is important to use `\"` inside the string,
        # as the string `["` is the separator for `foreach` in terraform.
        # In `count` it is just `[`
        idx_with_separator = f'\"{idx}\"' if isinstance(idx, str) else f'{idx}'
        new_block_id = f"{block.id}[{idx_with_separator}]"
        new_block_name = f"{block.name}[{idx_with_separator}]"

        if block.block_type == BlockType.MODULE:
            block.config[new_block_name] = block.config.pop(block.name)
        block.id = new_block_id
        block.name = new_block_name
        return idx_with_separator

    def _handle_static_statement(self, block_index: int, sub_graph: TerraformLocalGraph | None = None) -> \
            list[str] | dict[str, Any] | int | None:
        attrs = self.local_graph.vertices[block_index].attributes if not sub_graph \
            else sub_graph.vertices[block_index].attributes
        foreach_statement = attrs.get(FOREACH_STRING)
        count_statement = attrs.get(COUNT_STRING)
        if foreach_statement:
            return self._handle_static_foreach_statement(foreach_statement)
        if count_statement:
            return self._handle_static_count_statement(count_statement)
        return None

    def _handle_static_foreach_statement(self, statement: list[str] | dict[str, Any])\
            -> list[str] | dict[str, Any] | None:
        if isinstance(statement, list):
            statement = self.extract_from_list(statement)
        evaluated_statement = evaluate_terraform(statement)
        if isinstance(evaluated_statement, str):
            try:
                evaluated_statement = json.loads(evaluated_statement)
            except ValueError:
                pass
        if isinstance(evaluated_statement, set):
            evaluated_statement = list(evaluated_statement)
        if isinstance(evaluated_statement, (dict, list)) and all(isinstance(val, str) for val in evaluated_statement):
            return evaluated_statement
        return None

    def _handle_static_count_statement(self, statement: list[str] | int) -> int | None:
        if isinstance(statement, list):
            statement = self.extract_from_list(statement)
        evaluated_statement = evaluate_terraform(statement)
        if isinstance(evaluated_statement, int):
            return evaluated_statement
        return None

    def _is_static_foreach_statement(self, statement: str | list[str] | dict[str, Any]) -> bool:
        if isinstance(statement, list):
            if len(statement) == 1 and not statement[0]:
                return True
            statement = self.extract_from_list(statement)
        if isinstance(statement, str) and re.search(REFERENCES_VALUES, statement):
            return False
        if isinstance(statement, (list, dict)):
            result = True
            for s in statement:
                result &= self._is_static_foreach_statement(s)
            return result
        return True

    def _is_static_count_statement(self, statement: list[str] | int) -> bool:
        if isinstance(statement, list):
            statement = self.extract_from_list(statement)
        if isinstance(statement, int):
            return True
        if isinstance(statement, str) and not re.search(REFERENCES_VALUES, statement):
            return True
        return False

    def _is_static_statement(self, block_index: int, sub_graph: TerraformLocalGraph | None = None) -> bool:
        """
        foreach statement can be list/map of strings or map, if its string we need to render it for sure.
        """
        block = self.local_graph.vertices[block_index] if not sub_graph else sub_graph.vertices[block_index]
        foreach_statement = evaluate_terraform(block.attributes.get(FOREACH_STRING))
        count_statement = evaluate_terraform(block.attributes.get(COUNT_STRING))
        if foreach_statement:
            return self._is_static_foreach_statement(foreach_statement)
        if count_statement:
            return self._is_static_count_statement(count_statement)
        return False

    @staticmethod
    def extract_from_list(val: Any) -> Any:
        return val[0] if len(val) == 1 and isinstance(val[0], (str, int, list)) else val

    @staticmethod
    def need_to_add_quotes(code: str, key: str) -> bool:
        patterns = [r'lower\(' + key + r'\)', r'upper\(' + key + r'\)']
        for pattern in patterns:
            if re.search(pattern, code):
                return True
        if f'[{key}]' in code:
            return True
        return False



# -*- coding: utf-8 -*-
"""
============
stopit.utils
============

Misc utilities and common resources
"""

from __future__ import annotations

import functools
import logging
from logging import NullHandler
from typing import TYPE_CHECKING, Any, TypeVar, Callable, cast

from typing_extensions import ParamSpec, Self

if TYPE_CHECKING:
    from types import TracebackType

T = TypeVar("T")
P = ParamSpec("P")

# Custom logger
LOG = logging.getLogger(name='stopit')
LOG.addHandler(NullHandler())


class TimeoutException(Exception):
    """Raised when the block under context management takes longer to complete
    than the allowed maximum timeout value.
    """
    pass


class BaseTimeout:
    """Context manager for limiting in the time the execution of a block

    :param seconds: ``float`` or ``int`` duration enabled to run the context
      manager block
    :param swallow_exc: ``False`` if you want to manage the
      ``TimeoutException`` (or any other) in an outer ``try ... except``
      structure. ``True`` (default) if you just want to check the execution of
      the block with the ``state`` attribute of the context manager.
    """

    def __init__(self, seconds: int, swallow_exc: bool = True) -> None:

        # Possible values for the ``state`` attribute, self explanative
        self.EXECUTED, self.EXECUTING, self.TIMED_OUT, self.INTERRUPTED, self.CANCELED = range(5)

        self.seconds = seconds
        self.swallow_exc = swallow_exc
        self.state = self.EXECUTED

    def __bool__(self) -> bool:
        return self.state in (self.EXECUTED, self.EXECUTING, self.CANCELED)

    def __repr__(self) -> str:
        """Debug helper
        """
        return f"<{self.__class__.__name__} in state: {self.state}>"

    def __enter__(self) -> Self:
        self.state = self.EXECUTING
        self.setup_interrupt()
        return self

    def __exit__(self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: TracebackType | None) -> bool:
        if exc_type is TimeoutException:
            if self.state != self.TIMED_OUT:
                self.state = self.INTERRUPTED
                self.suppress_interrupt()
            LOG.warning(
                f"Code block execution exceeded {self.seconds} seconds timeout",
                exc_info=(exc_type, exc_val, exc_tb),
            )
            return self.swallow_exc
        else:
            if exc_type is None:
                self.state = self.EXECUTED
            self.suppress_interrupt()
        return False

    def cancel(self) -> None:
        """In case in the block you realize you don't need anymore
       limitation"""
        self.state = self.CANCELED
        self.suppress_interrupt()

    # Methods must be provided by subclasses
    def suppress_interrupt(self) -> None:
        """Removes/neutralizes the feature that interrupts the executed block
        """
        raise NotImplementedError

    def setup_interrupt(self) -> None:
        """Installs/initializes the feature that interrupts the executed block
        """
        raise NotImplementedError


class base_timeoutable:
    """A base for function or method decorator that raises a ``TimeoutException`` to
    decorated functions that should not last a certain amount of time.

    Any decorated callable may receive a ``timeout`` optional parameter that
    specifies the number of seconds allocated to the callable execution.

    The decorated functions that exceed that timeout return ``None`` or the
    value provided by the decorator.

    :param default: The default value in case we timed out during the decorated
      function execution. Default is None.

    :param timeout_param: As adding dynamically a ``timeout`` named parameter
      to the decorated callable may conflict with the callable signature, you
      may choose another name to provide that parameter. Your decoration line
      could look like ``@timeoutable(timeout_param='my_timeout')``

    .. note::

       This is a base class that must be subclassed. subclasses must override
       thz ``to_ctx_mgr`` with a timeout  context manager class which in turn
       must subclasses of above ``BaseTimeout`` class.
    """

    def __init__(self, default: Any = None, timeout_param: str = 'timeout') -> None:
        self.to_ctx_mgr: "type[BaseTimeout] | None" = None
        self.default, self.timeout_param = default, timeout_param

    def __call__(self, func: Callable[P, T]) -> Callable[P, T | Any]:
        @functools.wraps(func)
        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T | Any:
            timeout = cast(int, kwargs.pop(self.timeout_param, 0))
            if timeout:
                if not self.to_ctx_mgr:
                    return self.default

                with self.to_ctx_mgr(timeout, swallow_exc=True):
                    result = self.default
                    # ``result`` may not be assigned below in case of timeout
                    result = func(*args, **kwargs)
                return result
            else:
                return func(*args, **kwargs)
        return wrapper



from checkov.terraform.checks.resource.ncp.NACLInboundCheck import NACLInboundCheck


class NACLInbound21(NACLInboundCheck):
    def __init__(self):
        super().__init__(check_id="CKV_NCP_9", port=21)


check = NACLInbound21()



import re
from typing import Any, Dict, Tuple


class K8sValidator:
    name_disallowed_chars = re.compile("[#{}]+")  # noqa: CCE003  # a static attribute

    @staticmethod
    def is_valid_template(template: Any) -> Tuple[bool, str]:
        is_valid = isinstance(template, dict)
        if not is_valid:
            return is_valid, f'template is not a dict, but {type(template)}'

        is_valid, reason = K8sValidator._has_required_fields(template)
        if not is_valid:
            return is_valid, reason

        is_valid, reason = K8sValidator._is_section_valid(template)
        if not is_valid:
            return is_valid, reason

        return True, ''

    @staticmethod
    def _has_required_fields(template: Dict[str, Any]) -> Tuple[bool, str]:
        for key in ['apiVersion', 'kind']:
            if key not in template.keys():
                return False, f'the key {key} does not exist in template structure'
        return True, ''

    @staticmethod
    def _is_section_valid(template: Dict[str, Any]) -> Tuple[bool, str]:
        for segment in template.keys():
            if isinstance(template[segment], dict):
                is_section_valid, reason = K8sValidator._is_section_valid(template[segment])
                if not is_section_valid:
                    return is_section_valid, reason
            elif isinstance(template[segment], list):
                for entry in template[segment]:
                    if isinstance(entry, dict):
                        is_section_valid, reason = K8sValidator._is_section_valid(entry)
                        if not is_section_valid:
                            return is_section_valid, reason
            elif segment == 'name':
                is_section_valid, reason = K8sValidator._is_name_valid(template[segment])
                if not is_section_valid:
                    return is_section_valid, reason
        return True, ''

    @staticmethod
    def _is_name_valid(name: Any) -> Tuple[bool, str]:
        if not isinstance(name, str) or len(name) > 253:
            return False, f'name {name} is invalid'
        if K8sValidator.name_disallowed_chars.search(name):
            return False, f'name {name} is invalid'
        return True, ''



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ElasticBeanstalkUseEnhancedHealthChecks(BaseResourceCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 CA-7, NIST.800-53.r5 SI-2
        """
        name = "Ensure Elastic Beanstalk environments have enhanced health reporting enabled"
        id = "CKV_AWS_312"
        supported_resources = ('aws_elastic_beanstalk_environment',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        settings = conf.get("setting")
        if settings and isinstance(settings, list):
            if isinstance(settings[0], list):
                settings = settings[0]
            for setting in settings:
                namespace = setting.get("namespace")
                if isinstance(namespace, list) and namespace[0] == "aws:elasticbeanstalk:healthreporting:system":
                    name = setting.get("name")
                    if isinstance(name, list) and name[0] == "HealthStreamingEnabled":
                        value = setting.get("value")
                        if value and isinstance(value, list):
                            value = value[0]
                            if value == "True" or (value and isinstance(value, bool)):
                                return CheckResult.PASSED
        return CheckResult.FAILED


check = ElasticBeanstalkUseEnhancedHealthChecks()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class RDSInstanceAutoUpgrade(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure RDS instance auto upgrades for minor versions"
        id = "CKV_ALI_30"
        supported_resources = ("alicloud_db_instance",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "auto_upgrade_minor_version"

    def get_expected_value(self) -> Any:
        return "Auto"


check = RDSInstanceAutoUpgrade()



from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class EtcdCertAndKey(BaseK8sContainerCheck):
    def __init__(self) -> None:
        # CIS-1.6 2.1
        id = "CKV_K8S_116"
        name = "Ensure that the --cert-file and --key-file arguments are set as appropriate"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        if conf.get("command"):
            if "etcd" in conf["command"]:
                hasCertCommand = False
                hasKeyCommand = False
                for command in conf["command"]:
                    if command.startswith("--cert-file"):
                        hasCertCommand = True
                    elif command.startswith("--key-file"):
                        hasKeyCommand = True
                    if hasCertCommand and hasKeyCommand:
                        return CheckResult.PASSED
                return CheckResult.FAILED

        return CheckResult.PASSED


check = EtcdCertAndKey()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class DataprocPublicIpCluster(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Dataproc Clusters do not have public IPs"
        id = "CKV_GCP_103"
        supported_resources = ("google_dataproc_cluster",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "cluster_config/[0]/gce_cluster_config/[0]/internal_ip_only"


check = DataprocPublicIpCluster()



from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ElasticsearchDefaultSG(BaseResourceValueCheck):

    def __init__(self):
        """
        If an ES cluster is does not have its Security group specified it tries to use the default SG
        and that can never be right.
        """
        name = "Ensure that Elasticsearch is not using the default Security Group"
        id = "CKV_AWS_248"
        supported_resources = ['aws_elasticsearch_domain', 'aws_opensearch_domain']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "vpc_options/[0]/security_group_ids"

    def get_expected_value(self):
        return ANY_VALUE


check = ElasticsearchDefaultSG()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AMIEncryptionWithCMK(BaseResourceCheck):
    def __init__(self):
        name = "Ensure AMIs are encrypted using KMS CMKs"
        id = "CKV_AWS_204"
        supported_resources = ['aws_ami']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get('ebs_block_device'):
            mappings = conf.get('ebs_block_device')
            for mapping in mappings:
                if not mapping.get("snapshot_id"):
                    if not mapping.get("encrypted"):
                        return CheckResult.FAILED
                    if mapping.get("encrypted")[0] is False:
                        return CheckResult.FAILED
        # pass thru
        return CheckResult.PASSED


check = AMIEncryptionWithCMK()



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Callable, Dict, TypeVar, Set, Union
from typing_extensions import TypeAlias, TypedDict

if TYPE_CHECKING:
    from checkov.common.bridgecrew.severities import Severity
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.graph.db_connectors.db_connector import DBConnector
    from checkov.common.models.enums import CheckResult
    from checkov.common.runners.base_runner import BaseRunner  # noqa
    from networkx import DiGraph
    from igraph import Graph
    from checkov.terraform.modules.module_objects import TFDefinitionKey

_BaseRunner = TypeVar("_BaseRunner", bound="BaseRunner[Any]")

_ScannerCallableAlias: TypeAlias = Callable[
    [str, "BaseCheck", "list[_SkippedCheck]", "dict[str, Any]", str, str, "dict[str, Any]"], None
]

_Resource: TypeAlias = str
_Attributes: TypeAlias = Set[str]
ResourceAttributesToOmit: TypeAlias = Dict[_Resource, _Attributes]
LibraryGraph: TypeAlias = "Union[DiGraph, Graph]"
LibraryGraphConnector: TypeAlias = "Union[DBConnector[DiGraph], DBConnector[Graph]]"
TFDefinitionKeyType: TypeAlias = "Union[str, TFDefinitionKey]"


class _CheckResult(TypedDict, total=False):
    result: "CheckResult" | tuple["CheckResult", dict[str, Any]]
    suppress_comment: str
    evaluated_keys: list[str]
    results_configuration: dict[str, Any] | None
    check: BaseCheck
    entity: dict[str, Any]  # only exists for graph results


class _SkippedCheck(TypedDict, total=False):
    bc_id: str | None
    id: str
    suppress_comment: str
    line_number: int | None


class _ScaSuppressionsMaps(TypedDict, total=False):
    cve_suppresion_by_cve_map: dict[str, _SuppressedCves]
    licenses_suppressions_by_policy_and_package_map: dict[str, _SuppressedLicenses]


# _ScaSuppressions fields are in camel case because this is the output of the server report
class _ScaSuppressions(TypedDict, total=False):
    cves: _CvesSuppressions
    licenses: _LicensesSuppressions


class _CvesSuppressions(TypedDict):
    byCve: list[_SuppressedCves]


class _LicensesSuppressions(TypedDict):
    byPackage: list[_SuppressedLicenses]


class _SuppressedCves(TypedDict):
    reason: str
    cveId: str


class _SuppressedLicenses(TypedDict):
    reason: str
    packageName: str
    licensePolicy: str
    licenses: list[str]


class _BaselineFinding(TypedDict):
    resource: str
    check_ids: list[str]


class _BaselineFailedChecks(TypedDict):
    file: str
    findings: list[_BaselineFinding]


class _ReducedScanReport(TypedDict):
    checks: _ReducedScanReportCheck
    image_cached_results: list[dict[str, Any]]


class _ReducedScanReportCheck(TypedDict):
    failed_checks: list[dict[str, Any]]
    passed_checks: list[dict[str, Any]]
    skipped_checks: list[dict[str, Any]]


class _CicdDetails(TypedDict, total=False):
    commit: str | None
    pr: str | None
    runId: str | None
    scaCliScanId: str | None


class _ExitCodeThresholds(TypedDict):
    soft_fail: bool
    soft_fail_checks: list[str]
    soft_fail_threshold: Severity | None
    hard_fail_checks: list[str]
    hard_fail_threshold: Severity | None


class _ScaExitCodeThresholds(TypedDict):
    LICENSES: _ExitCodeThresholds
    VULNERABILITIES: _ExitCodeThresholds


class _LicenseStatus(TypedDict):
    package_name: str
    package_version: str
    policy: str
    license: str
    status: str


class _LicenseStatusWithLines(_LicenseStatus):
    lines: list[int] | None  # noqa: CCE003  # a static attribute


class _EntityContext(TypedDict, total=False):
    start_line: int
    end_line: int
    policy: str
    code_lines: list[tuple[int, str]]
    skipped_checks: list[_SkippedCheck]
    origin_relative_path: str



from __future__ import annotations

import logging
import os
from typing import Optional, Dict, Mapping, Set, Tuple, Callable, Any, List

import deep_merge

from checkov.common.runners.base_runner import filter_ignored_paths, IGNORE_HIDDEN_DIRECTORY_ENV, strtobool
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR, RESOLVED_MODULE_ENTRY_NAME
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.type_forcers import force_list
from checkov.common.variables.context import EvaluationContext
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.module import Module
from checkov.terraform.module_loading.content import ModuleContent
from checkov.terraform.module_loading.module_finder import load_tf_modules
from checkov.terraform.module_loading.registry import module_loader_registry as default_ml_registry, \
    ModuleLoaderRegistry
from checkov.common.util.parser_utils import get_tf_definition_key_from_module_dependency, \
    TERRAFORM_NESTED_MODULE_PATH_ENDING, is_acceptable_module_param
from checkov.terraform.modules.module_utils import load_or_die_quietly, safe_index, \
    remove_module_dependency_from_path, get_module_dependency_map, get_module_dependency_map_support_nested_modules, \
    clean_parser_types, serialize_definitions


def _filter_ignored_paths(root: str, paths: list[str], excluded_paths: list[str] | None) -> None:
    filter_ignored_paths(root, paths, excluded_paths)
    for path in force_list(paths):
        if path == default_ml_registry.external_modules_folder_name:
            paths.remove(path)


class Parser:
    def __init__(self, module_class: type[Module] = Module) -> None:
        self.module_class = module_class
        self._parsed_directories: set[str] = set()
        self.external_modules_source_map: Dict[Tuple[str, str], str] = {}
        self.module_address_map: Dict[Tuple[str, str], str] = {}
        self.loaded_files_map = {}

        # This ensures that we don't try to double-load modules
        # Tuple is <file>, <module_index>, <name> (see _load_modules)
        self._loaded_modules: Set[Tuple[str, int, str]] = set()
        self.external_variables_data = []
        self.enable_nested_modules = strtobool(os.getenv('CHECKOV_ENABLE_NESTED_MODULES', 'True'))

    def _init(self, directory: str, out_definitions: Optional[Dict],
              out_evaluations_context: Dict[str, Dict[str, EvaluationContext]],
              out_parsing_errors: Dict[str, Exception],
              env_vars: Mapping[str, str],
              download_external_modules: bool,
              external_modules_download_path: str,
              excluded_paths: Optional[List[str]] = None,
              tf_var_files: Optional[List[str]] = None):
        self.directory = directory
        self.out_definitions = out_definitions
        self.out_evaluations_context = out_evaluations_context
        self.out_parsing_errors = out_parsing_errors
        self.env_vars = env_vars
        self.download_external_modules = download_external_modules
        self.external_modules_download_path = external_modules_download_path
        self.external_modules_source_map = {}
        self.module_address_map = {}
        self.tf_var_files = tf_var_files
        self.dirname_cache = {}

        if self.out_evaluations_context is None:
            self.out_evaluations_context = {}
        if self.out_parsing_errors is None:
            self.out_parsing_errors = {}
        if self.env_vars is None:
            self.env_vars = dict(os.environ)
        self.excluded_paths = excluded_paths
        self.visited_definition_keys = set()
        self.module_to_resolved = {}
        self.keys_to_remove = set()

    def _check_process_dir(self, directory: str) -> bool:
        if directory not in self._parsed_directories:
            self._parsed_directories.add(directory)
            return True
        else:
            return False

    def parse_directory(self, directory: str, out_definitions: Optional[Dict],
                        out_evaluations_context: Dict[str, Dict[str, EvaluationContext]] = None,
                        out_parsing_errors: Dict[str, Exception] = None,
                        env_vars: Mapping[str, str] = None,
                        download_external_modules: bool = False,
                        external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
                        excluded_paths: Optional[List[str]] = None,
                        vars_files: Optional[List[str]] = None,
                        external_modules_content_cache: Optional[Dict[str, ModuleContent]] = None):
        self._init(directory, out_definitions, out_evaluations_context, out_parsing_errors, env_vars,
                   download_external_modules, external_modules_download_path, excluded_paths)
        self._parsed_directories.clear()
        default_ml_registry.root_dir = directory
        default_ml_registry.download_external_modules = download_external_modules
        default_ml_registry.external_modules_folder_name = external_modules_download_path
        default_ml_registry.module_content_cache = external_modules_content_cache if external_modules_content_cache else {}
        load_tf_modules(directory)
        self._parse_directory(dir_filter=lambda d: self._check_process_dir(d), vars_files=vars_files)
        if self.enable_nested_modules:
            self._update_resolved_modules()

    def parse_file(self, file: str, parsing_errors: Optional[Dict[str, Exception]] = None) -> Optional[Dict[str, Any]]:
        if file.endswith(".tf") or file.endswith(".tf.json") or file.endswith(".hcl"):
            parse_result = load_or_die_quietly(file, parsing_errors)
            if parse_result:
                parse_result = serialize_definitions(parse_result)
                parse_result = clean_parser_types(parse_result)
                return parse_result

        return None

    def _parse_directory(self, include_sub_dirs: bool = True,
                         module_loader_registry: ModuleLoaderRegistry = default_ml_registry,
                         dir_filter: Callable[[str], bool] = lambda _: True,
                         vars_files: Optional[List[str]] = None) -> None:
        """
    Load and resolve configuration files starting in the given directory, merging the
    resulting data into `tf_definitions`. This loads data according to the Terraform Code Organization
    specification (https://www.terraform.io/docs/configuration/index.html#code-organization), starting
    in the given directory and possibly moving out from there.

    The resulting data dictionary generally follows the layout of HCL parsing with a couple distinctions:
    - Data is broken out by file from which the data was loaded. So: <file>: <data>
      - Loaded modules will also be keyed by referrer info: <file>[<referring_file>#<index>]: <data>
    - Module block will included a "__resolved__" key with a list of the file/referrer names under
      which data for the file was loaded. For example: "__resolved__": ["main.tf#0"]. The values will
      correspond to the file names mentioned in the first bullet.
    - All variables that can be resolved will be resolved.


        :param include_sub_dirs:           If true, subdirectories will be walked.

        :param module_loader_registry:     Registry used for resolving modules. This allows customization of how
                                           much resolution is performed (and easier testing) by using a manually
                                           constructed registry rather than the default.
        :param dir_filter:                 Determines whether or not a directory should be processed. Returning
                                           True will allow processing. The argument will be the absolute path of
                                           the directory.
        """
        keys_referenced_as_modules: Set[str] = set()

        if include_sub_dirs:
            for sub_dir, d_names, _ in os.walk(self.directory):
                # filter subdirectories for future iterations (we filter files while iterating the directory)
                _filter_ignored_paths(sub_dir, d_names, self.excluded_paths)
                if dir_filter(os.path.abspath(sub_dir)):
                    self._internal_dir_load(sub_dir, module_loader_registry, dir_filter,
                                            keys_referenced_as_modules, vars_files=vars_files,
                                            root_dir=self.directory, excluded_paths=self.excluded_paths)
        else:
            self._internal_dir_load(self.directory, module_loader_registry, dir_filter,
                                    keys_referenced_as_modules, vars_files=vars_files)

        # Ensure anything that was referenced as a module is removed
        for key in keys_referenced_as_modules:
            if key in self.out_definitions:
                del self.out_definitions[key]

    def _internal_dir_load(self, directory: str,
                           module_loader_registry: ModuleLoaderRegistry,
                           dir_filter: Callable[[str], bool],
                           keys_referenced_as_modules: Set[str],
                           specified_vars: Optional[Mapping[str, str]] = None,
                           vars_files: Optional[List[str]] = None,
                           root_dir: Optional[str] = None,
                           excluded_paths: Optional[List[str]] = None,
                           nested_modules_data=None):
        """
    See `parse_directory` docs.
        :param directory:                  Directory in which .tf and .tfvars files will be loaded.
        :param module_loader_registry:     Registry used for resolving modules. This allows customization of how
                                       much resolution is performed (and easier testing) by using a manually
                                       constructed registry rather than the default.
        :param dir_filter:                 Determines whether or not a directory should be processed. Returning
                                       True will allow processing. The argument will be the absolute path of
                                       the directory.
        :param specified_vars:     Specifically defined variable values, overriding values from any other source.
        """

        # Stage 1: Look for applicable files in the directory:
        #          https://www.terraform.io/docs/configuration/index.html#code-organization
        #          Load the raw data for non-variable files, but perform no processing other than loading
        #          variable default values.
        #          Variable files are also flagged for later processing.
        var_value_and_file_map: Dict[str, Tuple[Any, str]] = {}
        hcl_tfvars: Optional[os.DirEntry] = None
        json_tfvars: Optional[os.DirEntry] = None
        auto_vars_files: List[os.DirEntry] = []  # *.auto.tfvars / *.auto.tfvars.json
        explicit_var_files: List[os.DirEntry] = []  # files passed with --var-file; only process the ones that are in this directory

        dir_contents = list(os.scandir(directory))
        if excluded_paths or IGNORE_HIDDEN_DIRECTORY_ENV:
            filter_ignored_paths(directory, dir_contents, excluded_paths)

        tf_files_to_load = []
        for file in dir_contents:
            # Ignore directories and hidden files
            try:
                if not file.is_file():
                    continue
            except OSError:
                # Skip files that can't be accessed
                continue

            # Variable files
            # See: https://www.terraform.io/docs/configuration/variables.html#variable-definitions-tfvars-files
            if file.name == "terraform.tfvars.json":
                json_tfvars = file
            elif file.name == "terraform.tfvars":
                hcl_tfvars = file
            elif file.name.endswith(".auto.tfvars.json") or file.name.endswith(".auto.tfvars"):
                auto_vars_files.append(file)
            elif vars_files and file.path in vars_files:
                explicit_var_files.append(file)

            # Resource files
            elif file.name.endswith(".tf") or file.name.endswith('.hcl'):  # TODO: add support for .tf.json
                tf_files_to_load.append(file)

        files_to_data = self._load_files(tf_files_to_load)

        for file, data in sorted(files_to_data, key=lambda x: x[0]):
            if not data:
                continue
            self.out_definitions[file] = data

            # Load variable defaults
            #  (see https://www.terraform.io/docs/configuration/variables.html#declaring-an-input-variable)
            var_blocks = data.get("variable")
            if var_blocks and isinstance(var_blocks, list):
                for var_block in var_blocks:
                    if not isinstance(var_block, dict):
                        continue
                    for var_name, var_definition in var_block.items():
                        if not isinstance(var_definition, dict):
                            continue

                        default_value = var_definition.get("default")
                        if default_value is not None and isinstance(default_value, list):
                            self.external_variables_data.append((var_name, default_value[0], file))
                            var_value_and_file_map[var_name] = default_value[0], file

        # Stage 2: Load vars in proper order:
        #          https://www.terraform.io/docs/configuration/variables.html#variable-definition-precedence
        #          Defaults are loaded in stage 1.
        #          Then loading in this order with later taking precedence:
        #             - Environment variables
        #             - The terraform.tfvars file, if present.
        #             - The terraform.tfvars.json file, if present.
        #             - Any *.auto.tfvars or *.auto.tfvars.json files, processed in lexical order of
        #               their filenames.
        #          Overriding everything else, variables form `specified_vars`, which are considered
        #          directly set.
        for key, value in self.env_vars.items():  # env vars
            if not key.startswith("TF_VAR_"):
                continue
            var_value_and_file_map[key[7:]] = value, f"env:{key}"
            self.external_variables_data.append((key[7:], value, f"env:{key}"))
        if hcl_tfvars:  # terraform.tfvars
            data = load_or_die_quietly(hcl_tfvars, self.out_parsing_errors, clean_definitions=False)
            if data:
                var_value_and_file_map.update({k: (safe_index(v, 0), hcl_tfvars.path) for k, v in data.items()})
                self.external_variables_data.extend([(k, safe_index(v, 0), hcl_tfvars.path) for k, v in data.items()])
        if json_tfvars:  # terraform.tfvars.json
            data = load_or_die_quietly(json_tfvars, self.out_parsing_errors)
            if data:
                var_value_and_file_map.update({k: (v, json_tfvars.path) for k, v in data.items()})
                self.external_variables_data.extend([(k, v, json_tfvars.path) for k, v in data.items()])

        auto_var_files_to_data = self._load_files(auto_vars_files)
        for var_file, data in sorted(auto_var_files_to_data, key=lambda x: x[0]):
            if data:
                var_value_and_file_map.update({k: (v, var_file) for k, v in data.items()})
                self.external_variables_data.extend([(k, v, var_file) for k, v in data.items()])

        explicit_var_files_to_data = self._load_files(explicit_var_files)
        # it's possible that os.scandir returned the var files in a different order than they were specified
        for var_file, data in sorted(explicit_var_files_to_data, key=lambda x: vars_files.index(x[0])):
            if data:
                var_value_and_file_map.update({k: (v, var_file) for k, v in data.items()})
                self.external_variables_data.extend([(k, v, var_file) for k, v in data.items()])

        if specified_vars:  # specified
            var_value_and_file_map.update({k: (v, "manual specification") for k, v in specified_vars.items()})
            self.external_variables_data.extend([(k, v, "manual specification") for k, v in specified_vars.items()])

        # Stage 4: Load modules
        #          This stage needs to be done in a loop (again... alas, no DAG) because modules might not
        #          be loadable until other modules are loaded. This happens when parameters to one module
        #          depend on the output of another. For such cases, the base module must be loaded, then
        #          a parameter resolution pass needs to happen, then the second module can be loaded.
        #
        #          One gotcha is that we need to make sure we load all modules at some point, even if their
        #          parameters don't resolve. So, if we hit a spot where resolution doesn't change anything
        #          and there are still modules to be loaded, they will be forced on the next pass.
        force_final_module_load = False
        for i in range(0, 10):  # circuit breaker - no more than 10 loops
            logging.debug("Module load loop %d", i)

            # Stage 4a: Load eligible modules
            # Add directory to self._parsed_directories to avoid loading it as sub dir
            if self.enable_nested_modules:
                dir_filter(directory)
            has_more_modules = self._load_modules(directory, module_loader_registry, dir_filter,
                                                  keys_referenced_as_modules, force_final_module_load,
                                                  nested_modules_data=nested_modules_data)

            # Stage 4b: Variable resolution round 2 - now with (possibly more) modules
            made_var_changes = False
            if not has_more_modules:
                break  # nothing more to do
            elif not made_var_changes:
                # If there are more modules to load but no variables were resolved, then to a final module
                # load, forcing things through without complete resolution.
                force_final_module_load = True

    def _load_files(self, files: list[os.DirEntry]):
        def _load_file(file: os.DirEntry):
            parsing_errors = {}
            result = load_or_die_quietly(file, parsing_errors)
            # the exceptions type can un-pickleable
            for path, e in parsing_errors.items():
                parsing_errors[path] = e

            return (file.path, result), parsing_errors

        files_to_data = []
        files_to_parse = []
        for file in files:
            data = self.loaded_files_map.get(file.path)
            if data:
                files_to_data.append((file.path, data))
            else:
                files_to_parse.append(file)

        results = [_load_file(f) for f in files_to_parse]
        for result, parsing_errors in results:
            self.out_parsing_errors.update(parsing_errors)
            files_to_data.append(result)
            if result[0] not in self.loaded_files_map:
                self.loaded_files_map[result[0]] = result[1]
        return files_to_data

    def _load_modules(self, root_dir: str, module_loader_registry: ModuleLoaderRegistry,
                      dir_filter: Callable[[str], bool],
                      keys_referenced_as_modules: Set[str], ignore_unresolved_params: bool = False,
                      nested_modules_data=None) -> bool:
        """
        Load modules which have not already been loaded and can be loaded (don't have unresolved parameters).

        :param ignore_unresolved_params:    If true, not-yet-loaded modules will be loaded even if they are
                                            passed parameters that are not fully resolved.
        :return:                            True if there were modules that were not loaded due to unresolved
                                            parameters.
        """
        all_module_definitions = {}
        all_module_evaluations_context = {}
        skipped_a_module = False
        for file in list(self.out_definitions.keys()):
            # Don't process a file in a directory other than the directory we're processing. For example,
            # if we're down dealing with <top_dir>/<module>/something.tf, we don't want to rescan files
            # up in <top_dir>.
            if self.get_dirname(file) != root_dir:
                continue
            # Don't process a file reference which has already been processed
            if file.endswith(TERRAFORM_NESTED_MODULE_PATH_ENDING):
                continue

            file_data = self.out_definitions.get(file)
            if file_data is None:
                continue
            module_calls = file_data.get("module")
            if not module_calls or not isinstance(module_calls, list):
                continue

            for module_index, module_call in enumerate(module_calls):
                if not isinstance(module_call, dict):
                    continue

                # There should only be one module reference per outer dict, but... safety first
                for module_call_name, module_call_data in module_call.items():
                    if not isinstance(module_call_data, dict):
                        continue

                    if self.enable_nested_modules:
                        file_key = self.get_file_key_with_nested_data(file, nested_modules_data)
                        current_nested_data = (file_key, module_index, module_call_name)

                        resolved_loc_list = []
                        if current_nested_data in self.module_to_resolved:
                            resolved_loc_list = self.module_to_resolved[current_nested_data]
                        self.module_to_resolved[current_nested_data] = resolved_loc_list

                    module_address = (file, module_index, module_call_name)
                    if not self.enable_nested_modules:
                        if module_address in self._loaded_modules:
                            continue

                    # Variables being passed to module, "source" and "version" are reserved
                    specified_vars = {k: v[0] if isinstance(v, list) else v for k, v in module_call_data.items()
                                      if k != "source" and k != "version"}

                    if not ignore_unresolved_params:
                        has_unresolved_params = False
                        for k, v in specified_vars.items():
                            if not is_acceptable_module_param(v) or not is_acceptable_module_param(k):
                                has_unresolved_params = True
                                break
                        if has_unresolved_params:
                            skipped_a_module = True
                            continue
                    self._loaded_modules.add(module_address)

                    source = module_call_data.get("source")
                    if not source or not isinstance(source, list):
                        continue
                    source = source[0]
                    if not isinstance(source, str):
                        logging.debug(f"Skipping loading of {module_call_name} as source is not a string, it is: {source}")
                        continue
                    elif source in ['./', '.']:
                        logging.debug(f"Skipping loading of {module_call_name} as source is the current dir")
                        continue

                    # Special handling for local sources to make sure we aren't double-parsing
                    if source.startswith("./") or source.startswith("../"):
                        source = os.path.normpath(
                            os.path.join(os.path.dirname(remove_module_dependency_from_path(file)), source))

                    version = module_call_data.get("version", "latest")
                    if version and isinstance(version, list):
                        version = version[0]
                    try:
                        content = module_loader_registry.load(root_dir, source, version)
                        if not content.loaded():
                            logging.info(f'Got no content for {source}:{version}')
                            continue

                        new_nested_modules_data = {'module_index': module_index, 'file': file,
                                                   'nested_modules_data': nested_modules_data}

                        self._internal_dir_load(directory=content.path(),
                                                module_loader_registry=module_loader_registry,
                                                dir_filter=dir_filter, specified_vars=specified_vars,
                                                keys_referenced_as_modules=keys_referenced_as_modules,
                                                nested_modules_data=new_nested_modules_data)

                        module_definitions = {
                            path: definition
                            for path, definition in self.out_definitions.items()
                            if self.get_dirname(path) == content.path()
                        }

                        if not module_definitions:
                            continue

                        # NOTE: Modules are put into the main TF definitions structure "as normal" with the
                        #       notable exception of the file name. For loaded modules referrer information is
                        #       appended to the file name to create this format:
                        #         <file_name>[<referred_file>#<referrer_index>]
                        #       For example:
                        #         /the/path/module/my_module.tf[/the/path/main.tf#0]
                        #       The referrer and index allow a module allow a module to be loaded multiple
                        #       times with differing data.
                        #
                        #       In addition, the referring block will have a "__resolved__" key added with a
                        #       list pointing to the location of the module data that was resolved. For example:
                        #         "__resolved__": ["/the/path/module/my_module.tf[/the/path/main.tf#0]"]

                        if not self.enable_nested_modules:
                            resolved_loc_list = module_call_data.get(RESOLVED_MODULE_ENTRY_NAME)
                            if resolved_loc_list is None:
                                resolved_loc_list = []
                                module_call_data[RESOLVED_MODULE_ENTRY_NAME] = resolved_loc_list

                        # NOTE: Modules can load other modules, so only append referrer information where it
                        #       has not already been added.

                        keys = list(module_definitions.keys())
                        for key in keys:
                            if key.endswith(TERRAFORM_NESTED_MODULE_PATH_ENDING) or file.endswith(TERRAFORM_NESTED_MODULE_PATH_ENDING):
                                continue
                            keys_referenced_as_modules.add(key)
                            if self.enable_nested_modules:
                                new_key = self.get_new_nested_module_key(key, file, module_index, nested_modules_data)
                                if new_key in self.visited_definition_keys:
                                    del module_definitions[key]
                                    del self.out_definitions[key]
                                    continue
                            else:
                                new_key = get_tf_definition_key_from_module_dependency(key, file, module_index)
                            module_definitions[new_key] = module_definitions[key]
                            del module_definitions[key]
                            del self.out_definitions[key]
                            self.keys_to_remove.add(key)

                            if self.enable_nested_modules:
                                self.visited_definition_keys.add(new_key)
                            if new_key not in resolved_loc_list:
                                resolved_loc_list.append(new_key)
                            if (file, module_call_name) not in self.module_address_map:
                                self.module_address_map[(file, module_call_name)] = str(module_index)
                        resolved_loc_list.sort()  # For testing, need predictable ordering

                        if all_module_definitions:
                            deep_merge.merge(all_module_definitions, module_definitions)
                        else:
                            all_module_definitions = module_definitions

                        self.external_modules_source_map[(source, version)] = content.path()
                    except Exception as e:
                        logging.warning("Unable to load module (source=\"%s\" version=\"%s\"): %s",
                                        source, version, e)

        if all_module_definitions:
            deep_merge.merge(self.out_definitions, all_module_definitions)
        if all_module_evaluations_context:
            deep_merge.merge(self.out_evaluations_context, all_module_evaluations_context)
        return skipped_a_module

    def parse_hcl_module(
        self,
        source_dir: str,
        source: str,
        download_external_modules: bool = False,
        external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
        parsing_errors: dict[str, Exception] | None = None,
        excluded_paths: list[str] | None = None,
        vars_files: list[str] | None = None,
        external_modules_content_cache: dict[str, ModuleContent] | None = None,
        create_graph: bool = True,
    ) -> tuple[Module | None, dict[str, dict[str, Any]]]:
        tf_definitions: dict[str, dict[str, Any]] = {}
        self.parse_directory(directory=source_dir, out_definitions=tf_definitions, out_evaluations_context={},
                             out_parsing_errors=parsing_errors if parsing_errors is not None else {},
                             download_external_modules=download_external_modules,
                             external_modules_download_path=external_modules_download_path, excluded_paths=excluded_paths,
                             vars_files=vars_files, external_modules_content_cache=external_modules_content_cache)
        tf_definitions = clean_parser_types(tf_definitions)
        tf_definitions = serialize_definitions(tf_definitions)

        module = None
        if create_graph:
            module, tf_definitions = self.parse_hcl_module_from_tf_definitions(tf_definitions, source_dir, source)

        return module, tf_definitions

    def _remove_unused_path_recursive(self, path):
        self.out_definitions.pop(path, None)
        for key in list(self.module_to_resolved.keys()):
            file_key, module_index, module_name = key
            if path == file_key:
                for resolved_path in self.module_to_resolved[key]:
                    self._remove_unused_path_recursive(resolved_path)
                self.module_to_resolved.pop(key, None)

    def _update_resolved_modules(self):
        for key in list(self.module_to_resolved.keys()):
            file_key, module_index, module_name = key
            if file_key in self.keys_to_remove:
                for path in self.module_to_resolved[key]:
                    self._remove_unused_path_recursive(path)
                self.module_to_resolved.pop(key, None)

        for key, resolved_list in self.module_to_resolved.items():
            file_key, module_index, module_name = key
            if file_key not in self.out_definitions:
                continue
            self.out_definitions[file_key]['module'][module_index][module_name][RESOLVED_MODULE_ENTRY_NAME] = resolved_list

    def parse_hcl_module_from_tf_definitions(
        self,
        tf_definitions: Dict[str, Dict[str, Any]],
        source_dir: str,
        source: str,
    ) -> Tuple[Module, Dict[str, Dict[str, Any]]]:
        if self.enable_nested_modules:
            module_dependency_map, tf_definitions, dep_index_mapping = get_module_dependency_map_support_nested_modules(tf_definitions)
        else:
            module_dependency_map, tf_definitions, dep_index_mapping = get_module_dependency_map(tf_definitions)
        module = self.get_new_module(
            source_dir=source_dir,
            module_dependency_map=module_dependency_map,
            module_address_map=self.module_address_map,
            external_modules_source_map=self.external_modules_source_map,
            dep_index_mapping=dep_index_mapping,
        )
        self.add_tfvars(module, source)
        copy_of_tf_definitions = pickle_deepcopy(tf_definitions)
        for file_path, blocks in copy_of_tf_definitions.items():
            for block_type in blocks:
                try:
                    module.add_blocks(block_type, blocks[block_type], file_path, source)
                except Exception as e:
                    logging.warning(f'Failed to add block {blocks[block_type]}. Error:')
                    logging.warning(e, exc_info=False)
        return module, tf_definitions

    def get_file_key_with_nested_data(self, file, nested_data):
        if not nested_data:
            return file
        nested_str = self.get_file_key_with_nested_data(nested_data.get("file"), nested_data.get('nested_modules_data'))
        nested_module_index = nested_data.get('module_index')
        return get_tf_definition_key_from_module_dependency(file, nested_str, nested_module_index)

    def get_new_nested_module_key(self, key, file, module_index, nested_data) -> str:
        if not nested_data:
            return get_tf_definition_key_from_module_dependency(key, file, module_index)
        visited_key_to_add = get_tf_definition_key_from_module_dependency(key, file, module_index)
        self.visited_definition_keys.add(visited_key_to_add)
        nested_key = self.get_new_nested_module_key('', nested_data.get('file'),
                                                    nested_data.get('module_index'),
                                                    nested_data.get('nested_modules_data'))
        return get_tf_definition_key_from_module_dependency(key, f"{file}{nested_key}", module_index)

    def add_tfvars(self, module: Module, source: str) -> None:
        if not self.external_variables_data:
            return
        for (var_name, default, path) in self.external_variables_data:
            if ".tfvars" in path:
                block = {var_name: {"default": default}}
                module.add_blocks(BlockType.TF_VARIABLE, block, path, source)

    def get_dirname(self, path: str) -> str:
        dirname_path = self.dirname_cache.get(path)
        if not dirname_path:
            dirname_path = os.path.dirname(path)
            self.dirname_cache[path] = dirname_path
        return dirname_path

    @staticmethod
    def get_new_module(
            source_dir: str,
            module_dependency_map: dict[str, list[list[str]]],
            module_address_map: dict[tuple[str, str], str],
            external_modules_source_map: dict[tuple[str, str], str],
            dep_index_mapping: dict[tuple[str, str], list[str]],
    ) -> Module:
        return Module(
            source_dir=source_dir,
            module_dependency_map=module_dependency_map,
            module_address_map=module_address_map,
            external_modules_source_map=external_modules_source_map,
            dep_index_mapping=dep_index_mapping
        )



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class APIManagementBackendHTTPS(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure API management backend uses https"
        id = "CKV_AZURE_215"
        supported_resources = ("azurerm_api_management_backend",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["url"]
        url = conf.get("url")
        if url and isinstance(url, list):
            if "https" in url[0]:
                return CheckResult.PASSED

            return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = APIManagementBackendHTTPS()
